/*
  HConsumer - HTML parser
  */

#include "hconsumer.h"
#include <curl/curl.h>


#include <mozilla/mozilla_v2.h>
extern pParam_v2 * CurrentParam;


//#define _H_DEBUG

CString HConsumer::ReadText(cgiOutStream& CGIStream, int& curPos, const CString& iStr){
  int prevPos = curPos;
  while ((iStr[curPos]!='<')&&(iStr[curPos]!='&')&&(iStr[curPos])) curPos++;
  LastToken = iStr[curPos];
  if (prevPos != curPos) return iStr.Copy(prevPos, curPos-prevPos); else return "";
}

CString HConsumer::ReadToken(cgiOutStream& CGIStream, int& curPos, const CString& iStr){    
  if ((iStr[curPos+1] <= 32)&&(iStr[curPos+1]>0)) return ""; else curPos++;
  int prevPos = curPos;
  int Depth = 0;
  while (
	 ((iStr[curPos]!='>')&&(LastToken == '<')) || 
	 ((iStr[curPos]!=';')&&(LastToken == '&')) ||
	 Depth) {
    if (!iStr[curPos]) break;
    if (!isalnum(iStr[curPos]) && (iStr[curPos]!=';') && (LastToken == '&') && (iStr[curPos]!='#')) {
      curPos = prevPos;
      return "";
    }
    if (LastToken == '<') {
      switch (iStr[curPos]){               
      case '<':
	Depth++;
	break;
      case '>': 
	Depth--;
	break;
      }
    } else if ((LastToken == '&')&&(iStr[curPos]==';')) break;
    curPos++;
  }
  if ((iStr[curPos] == '>')&&(LastToken == '<')) curPos++;
  else if ((iStr[curPos] == ';')&&(LastToken == '&')) curPos++;
  CString Result = iStr.Copy(prevPos, curPos-prevPos-1);  
  return Result;
}

void HConsumer::SkipSpaces(cgiOutStream& CGIStream, int& curPos, const CString& Token){
  while ((Token[curPos]<=' ')&&(Token[curPos]>0)) curPos++;
}

void HConsumer::ParseToken(cgiOutStream& CGIStream, const CString& Token){ 
  if (LastToken == '&') {
    // http://www.w3.org/TR/REC-html40/sgml/entities.html#iso-88591        
    if (isShowing()) CGIStream << Token;
  } else {
    int curPos = 0;
    int prevPos = 0;      
    if (Token[0] == '/') curPos++;
    while ((Token[curPos] > ' ') && Token[curPos]) curPos++;
    CString Name = Token.Copy(prevPos, curPos - prevPos).StrTrim32();
#ifdef _H_DEBUG
    cout << "token name: [" << Name << "]" << endl;
#endif    
    if ((!UsefulTags.Count()) || (UsefulTags.Contains(Name))) {
      if (Name == "!--") {
	if (isShowing()) CGIStream << "<" << Token << ">";
	CString tToken = Token; tToken.StrDelete(0, strlen("!--")); tToken.StrTrim32();
	if (tToken.EndsWith("--")) tToken.StrDelete(tToken.StrLength() - strlen("--"), tToken.StrLength());
	tToken.StrTrim32();
	Comments+=tToken;
      } else {
	html_tag curTag(Name.asString(), 0);      
	while (curPos < Token.StrLength()) {
	  SkipSpaces(CGIStream, curPos, Token);
	  if (!Token[curPos]) break;
	  CString IName;
	  CString IValue;
	  ReadNameValue(CGIStream, curPos, Token, IName, IValue);
#ifdef _H_DEBUG
	  CGIStream << "name=[" << IName << "] value=[" << IValue << "]" << elf;
#endif       
	  curTag.add_value(IName, IValue);
	}
	onNewTag(CGIStream, curTag);
      }      
      if (FAction.StrLength()) {
	CGIStream << "\n<input type=\"hidden\" name=\"P-ACTION\" value=\"" << FAction << "\">\n";
	CGIStream << "<input type=\"hidden\" name=\"P-IDENT\" value=\"" << CurrentParam->get_entry("P-IDENT") << "\">\n";
	FAction.Free();
      }
    }
  }
}

CString HConsumer::ReadString(cgiOutStream& CGIStream, int& curPos, const CString& Token){
  SkipSpaces(CGIStream, curPos, Token);
  int prevPos = curPos;
  int Level = 0;
  while (curPos < Token.StrLength()) {    
    if ((Token[curPos] == '=')&&(!Level)) break;
    else if ((Token[curPos] <= ' ')&&(Token[curPos]>0)&&(!Level)) break;
    else if (Token[curPos] == '\"') {
      if (!Level) Level++; else {
	Level--;
	curPos++;
	break;
      }
    } 
    curPos++;
  }
  CString Result = Token.Copy(prevPos, curPos-prevPos);
  if (curPos < Token.StrLength()) while ((Token[curPos] <= ' ')&&(Token[curPos]>0)) curPos++;
  return Result.StrTrim32();
}

void HConsumer::ReadNameValue(cgiOutStream& CGIStream, int& curPos, const CString& Token, CString& Name, CString& Value){    
  Value.Free();
  Name = ReadString(CGIStream, curPos, Token);
  if (Token[curPos] == '=') {    
    while (Token[curPos] == '=') curPos++;
    Value = ReadString(CGIStream, curPos, Token);    
  }  
}

void HConsumer::HParse(cgiOutStream& CGIStream, const CString& iStr) {
  int curPos = 0;
  CString cText;
  while (curPos < iStr.StrLength()) {    
    cText = ReadText(CGIStream, curPos, iStr);    
    if (isShowing()) CGIStream << cText;
    if (iStr[curPos]) {
      CString HtmlToken = ReadToken(CGIStream, curPos, iStr);
      if (HtmlToken.StrLength()) {	
	ParseToken(CGIStream, HtmlToken);
      } else {
	if (isShowing()) CGIStream << (char) LastToken;	
	curPos++;
      }
    } else break;
  }  
}

HConsumer::HConsumer(int Operation, cgiOutStream& CGIStream, const CString& iStr){
  inForm = 0;
  inBody = 0;
  POperation = Operation;
  HParse(CGIStream, iStr);  
}

HConsumer::~HConsumer(void){
  
}

void HConsumer::Dequote(CString& iStr){
  if (iStr.StrLength()) {
    if (iStr[0] == '\"') iStr.StrDelete(0, 1);
    if (iStr[iStr.StrLength()-1] == '\"') iStr.StrDelete(iStr.StrLength()-1,1);
    iStr.StrTrim();
  }
}

void HConsumer::setOperation(int Value){
  POperation = Value;
}

int HConsumer::isShowing(void){
  if (POperation == P_EXTRACT_FORM) return inForm;
  else if (POperation == P_EXTRACT_BODY) return inBody;
  else return 1;
}

void HConsumer::onNewTag(cgiOutStream& CGIStream, html_tag& curTag){  
  /*
    curTag.name
    curTag.get_value("NAME")
    */
  if (curTag.name == "FORM") inForm = 1; else if (curTag.name == "/FORM") inForm = 0;
  else if (curTag.name == "BODY") inBody = 1; else if (curTag.name == "/BODY") inBody = 0;
  if (isShowing()) {
    CGIStream << '<' << curTag.name; 
    for (int i=0;i<curTag.entries_count();i++) {
      CGIStream << " ";
      if (curTag.get_name(i).StrLength()) CGIStream << curTag.get_name(i);
      if ((curTag.name == "FORM")&&(curTag.get_name(i) == "ACTION")) {
	FAction = curTag.get_value(i); Dequote(FAction);
	CGIStream << "=" << getenv("SCRIPT_NAME");
      } else if ((curTag.name == "IMG")&&(curTag.get_name(i) == "SRC")) {
	CUrl Url(CurrentParam->get_entry("P-IDENT")); 
	CString Value = curTag.get_value(i); Dequote(Value);
	CString Tmp(Url.GetHost());
	CGIStream << "=http://" << Tmp << Value;       
      } else if (curTag.get_value(i).StrLength()) CGIStream << '=' << curTag.get_value(i);
    }
    if (curTag.name == "!--") CGIStream << "-->";
    else CGIStream << ">";
  }  
}

