/*
  CDate.cpp
  */

#include "cdate.h"

const char * AMonthsFrench[] = { "Janvier", "Fevrier", "Mars", "Avril", 
								"Mai", "Juin", "Juillet", "Aot","Septembre","Octobre",
								"Novembre", "Decembre" };

const char * AMonthsEnglish[] = {"January","February","March","April",
								 "May","June","July","August","September","October",
								 "November","December"};
  
const char * ADaysFrench[] = {	"Lundi", "Mardi", "Mercredi", "Jeudi", "Vendredi", "Samedi", "Dimanche" };
const char * ADaysEnglish[] = {	"Monday", "Tuesday", "Wednesday", "Thursday", "Friday", "Saturday", "Sunday" };
const char * ADaysEnglishShort[] = {"Mon","Tue","Wed","Thu","Fri","Sat","Sun"};
const char * AMonthsEnglishShort[] = {"Jan","Feb","Mar","Apr", "May","Jun","Jul","Aug","Sep","Oct","Nov","Dec"};

const char * CDate::DayFrench(void){
  return DayFrench(DayOfWeek());
}

const char * CDate::MonthFrench(void){
  return MonthFrench(tm_time->tm_mon);
}

const char * CDate::AscTime(void){
  return asctime(tm_time);
}

const char * CDate::DayFrench(int Day){
	assert(Day < 7);
	return ADaysFrench[Day];  
}

const char * CDate::MonthFrench(int Month){
	assert(Month < 12);
	return AMonthsFrench[Month];
}

const char * CDate::DayEnglishShort(int Day){
  assert(Day < 7);
  return ADaysEnglishShort[Day];
}

const char * CDate::DayEnglish(int Day){
	assert(Day < 7);
	return ADaysEnglish[Day];  
}

const char * CDate::MonthEnglish(int Month){
  assert(Month < 12);
  return AMonthsEnglish[Month];
}

const char * CDate::MonthEnglishShort(int Month){
  assert(Month < 12);
  return AMonthsEnglishShort[Month];
}

int CDate::DayOfWeek (int year, int month, int day){
  static int t[] = {0, 3, 2, 5, 0, 3, 5, 1, 4, 6, 2, 4};
  year -= month < 3;
  return (year + year/4 - year/100 + year/400 + t[month-1] + day)%7;
}

void CDate::Protect(void){
   if (!tm_time){
     time(&time_t_time); 
     tm_time = localtime(&time_t_time);
   } 
}

CDate::CDate(void){
  time(&time_t_time); 
  tm_time = localtime(&time_t_time);
}

const char * CDate::DayEnglish(void){
  return DayEnglish(DayOfWeek());
}

const char * CDate::DayEnglishShort(void){
  return DayEnglishShort(DayOfWeek());
}

const char * CDate::MonthEnglish(void){
  //Protect();
  return MonthEnglish(tm_time->tm_mon);
}

const char * CDate::MonthEnglishShort(void){
  return MonthEnglishShort(tm_time->tm_mon);
}

int CDate::DayOfWeek(void){
  //Protect();
  return DayOfWeek(tm_time->tm_year, tm_time->tm_mon+1, tm_time->tm_mday);
}

CDate * CurrentCDate;

void GetVariableCDate(CString & Term){
  CurrentCDate->GetVariable(Term);
}

CString& CDate::Year(CString& Target){
  //Protect();
  Target.Free();  
  Target+=(tm_time->tm_year+1900);
  return Target;
}

CString& CDate::Month(CString& Target){
  //Protect();
  Target.Free();
  Target+=tm_time->tm_mon+1;
  return Target;
}

CString& CDate::Day(CString& Target){
  //Protect();
  Target.Free();
  Target+=tm_time->tm_mday;
  return Target;
}

void CDate::GetVariable(CString& Term) {
  //Protect();
  if (Term == "DayEnglish") Term = DayEnglish();
  else if (Term == "DayEng") Term = DayEnglishShort();
  else if (Term == "DayFrench") Term = DayFrench();
  else if (Term == "MonthFrench") Term = MonthFrench();
  else if (Term == "MonthEnglish") Term = MonthEnglish();
  else if (Term == "MonthEng") Term = MonthEnglishShort();
  else if (Term == "Year") Year(Term);
  else if (Term == "Day") Day(Term);
  else if (Term == "Month") Month(Term);
  else if (Term == "Hour") {char tmp[3]; sprintf(tmp, "%02d", tm_time->tm_hour); Term = tmp; }
  else if (Term == "Min") {char tmp[3]; sprintf(tmp, "%02d", tm_time->tm_min); Term = tmp; }
  else if (Term == "Sec") {char tmp[3]; sprintf(tmp, "%02d", tm_time->tm_sec); Term = tmp; }
}

CString CDate::Map(const CString& Expression) {
  CString Tmp;
  Map(Expression, Tmp);
  return Tmp;
}

CString& CDate::Map(const CString& Expression, CString& Target){
  CDate * OldCDate = CurrentCDate;
  CurrentCDate = this;
  Expression.MapTerm(&GetVariableCDate, Target);
  CurrentCDate = OldCDate;
  return Target;
}



