#include "ccontainer.h"

ostream& operator<<(ostream& os, const entry_manager& s){
  os << s.Singles.Count() << " single pairs" << "n";
  for (int i=0;i<s.Singles.Count();i++)
    os << "\t" << s.get_name(i) << "::" << s.get_value(i) << "n";
  os << s.Multiples.Count() << " recursive entries" << "n";
  for (int j=0;j<s.Multiples.Count();j++){
    os << "\t" << s.Multiples[j].Name << "n";
    for (int l=0;l<s.Multiples[j].Entries.Count();l++)
      os << "\t\t" << s.get_name(j, l) << "::" << s.get_value(j, l) << "n";
  }
  return os;
}

CEntryMultiple EmptyMultiple;

void entry_manager::clear(void){
  Singles.Clear();
  Multiples.Clear();
}

entry_manager::entry_manager(void) : Singles(""), Multiples(EmptyMultiple) {
  char * current_root = getcwd(0, MAXPATHLEN);
  if (current_root) {
    Root = current_root;
    delete current_root;
  }
}

entry_manager::entry_manager(entry_manager& s) : Singles(""), Multiples(EmptyMultiple) {
  Singles = s.Singles;
  Multiples = s.Multiples;
}

int entry_manager::find_multiple(const CString& class_name) const{
  return Multiples.Contains(class_name);
}

int entry_manager::find_single(const CString & class_name) const{
  return Singles.Contains(class_name);
}

entry_manager::~entry_manager(void){
}

void entry_manager::add_value(const CString& name, const CString& value){
  Singles.Add(name, value);
}

void entry_manager::set_value(const CString& name, const CString& value){
  set_value(name, value, 0, 0);
}

void entry_manager::set_value(const CString& name, const CString& value, int fail, int warning){
  Singles.Set(name, value, fail, warning);
}

CString entry_manager::get_value(const CString& name) const {
  return Singles.Search(name);
}

CString  entry_manager::get_value(int index) const{
  return Singles.Get(index);
}

CString  entry_manager::get_name(int index) const{
  return Singles.GetKey(index);
}

void entry_manager::del_value(const CString& class_name){
  int ndx = Multiples.Contains(class_name);  
  if (ndx >= 0) Multiples.DelHash(ndx);  
}

void entry_manager::add_value(const CString& class_name, const CString& name, const CString& value){
  CEntryMultiple * Entry = Multiples.SearchRef(class_name);
  if (!Entry) {
    CEntryMultiple CurrentEntry(class_name);
    Entry = Multiples.Add(class_name, CurrentEntry);
  }
  if (!Entry) {    
    //cgi_error("entry_manager::add_value() failed at CHashTable<CEntryMultiple>.Add()");
  } else {
    Entry->Entries.Add(name, value);
  }
}

void entry_manager::set_value(const CString& class_name, const CString& name, const CString& value, int fail, int warning){
  CEntryMultiple * Entry = Multiples.SearchRef(class_name);
  if (!Entry) {
    //if (fail) cgi_error("entry_manager::set_value() - failed (was fatal)");
    //if (warning) cgiOut << "entry_manager::set_value() - [" << class_name << "] class not found" << "\n";
    CEntryMultiple CurrentEntry(class_name);
    Entry = Multiples.Add(class_name, CurrentEntry);
  }
  Entry->Entries.Set(name, value, fail, warning);
}

void entry_manager::set_value(const CString& class_name, const CString& name, const CString& value){
  set_value(class_name, name, value, 0, 0);
}

CString entry_manager::get_value(int class_index, int index) const {
  return Multiples[class_index].Entries.Get(index);
}

CString entry_manager::get_value(const CString& class_name, const CString& name) const{
  CEntryMultiple * Entry = Multiples.SearchRef(class_name);
  if (!Entry) return "";
  return Entry->Entries.Search(name);
}

CString entry_manager::get_value(const CString& class_name, int index) const{
  CEntryMultiple * Entry = Multiples.SearchRef(class_name);
  if (!Entry) return "";
  return Entry->Entries.Get(index);
}

CString  entry_manager::get_name(int class_index, int index) const{
  return Multiples[class_index].Entries.GetKey(index);
}

CString  entry_manager::get_name(const CString& class_name, int index) const{
  CEntryMultiple * Entry = Multiples.SearchRef(class_name);
  if (!Entry) return "";
  return Entry->Entries.GetKey(index);
}
  
CString  entry_manager::get_class(int index) const{
  return Multiples[index].Name;
}

int  entry_manager::class_count(void){
  return Multiples.Count();
}

int  entry_manager::class_count(int class_index){
  return Multiples[class_index].Entries.Count();
}

int  entry_manager::class_count(const CString& class_name){
  CEntryMultiple * Entry = Multiples.SearchRef(class_name);
  if (!Entry) return 0; 
  else return Entry->Entries.Count();
}
  
int entry_manager::entries_count(void) {
  return Singles.Count();
}

CString entry_manager::make_equiv_path(const CString& structure){
  CString retStr = Root;
  retStr.remove_backslash();
#ifdef ANSI
  retStr+="/equiv/";
#else
  retStr+="\\equiv\\";
#endif
  retStr+=structure.asString();
  return retStr;
}



