/*
  CUrl.cpp - URL implementation 
  (c) Daniel Doubrovkine - dblock@vestris.com
  */

#include "curl.h"

//#define _U_DEBUG

CUrl::CUrl(void){
  Valid = 0;
}

CUrl::CUrl(const CString& iUrl){
  SetUrl(iUrl);
}

void CUrl::SetUrl(const CString& iUrl){
  if (UBrute.StrLength()) {
    UScheme.Free();
    USpecific.Free();
    UUser.Free();
    UPassword.Free();
    UPort.Free();
    UHost.Free();
    UUrlPath.Free();  
    UHostVector.Clear();
    UFtpPath.Clear();
    UFtpPathType = 0;    
    UHttpPath.Free();
    UHttpSearchPath.Free();
    UHttpArg.Free();
    UHttpSig.Free();    
  }
  UBrute = iUrl;  
  Valid = 0;
  ParseUrl(UBrute);
}

CString CUrl::ReadScheme(int& curPos, const CString& iUrl){
  /*
    scheme must start at zero, known schemes are
    (implemented) ftp       File Transfer protocol
    (implemented) http      Hypertext Transfer Protocol
    gopher                  The Gopher protocol
    mailto                  Electronic mail address
    news                    USENET news
    nntp                    USENET news using NNTP access
    telnet                  Reference to interactive sessions
    wais                    Wide Area Information Servers
    file                    Host-specific file names
    prospero                Prospero Directory Service
    */
  int curPosLocal = 0;
  while (curPosLocal < iUrl.StrLength()) {
    switch(iUrl[curPosLocal]) {   
    case ':':
      curPos = curPosLocal+1;     
      return iUrl.Copy(0, curPosLocal);
    case '+':
    case '.':
    case '-': 
      break;
    default:
      if ((iUrl[curPosLocal] >= 'a')&&(iUrl[curPosLocal] <= 'z')) break; else return "";
    }
    curPosLocal++;
  }
  return "";
}

void CUrl::ParseUrl(const CString& iUrl){
  // URL general syntax: <scheme>:<scheme-specific-part>
  int curPos = 0;
  UScheme = ReadScheme(curPos, iUrl);
#ifdef _U_DEBUG
  cout << "scheme:   " << UScheme << endl;
#endif
  if (UScheme.StrLength()) {
    if (iUrl.Copy(curPos, 2) != "//") {
      UScheme = "http";
      curPos = 0;
    } else curPos+=2;
  } else {
    if (!UScheme.StrLength()) UScheme = "http";
  }
  USpecific = iUrl.Copy(curPos, iUrl.StrLength());
  /*
    <user>:<password>@<host>:<port>/<url-path>
    parse terms of dilimiters
    */  
  if (ParseUrlSpecific(curPos, USpecific)) {
    if (!UHost.StrLength()) UHost = "localhost";
    if (!UPort.StrLength()) UPort = "80";
#ifdef _U_DEBUG
    cout << "user:     " << UUser << endl;
    cout << "password: " << UPassword << endl;
    cout << "host:     " << UHost << endl;
    cout << "port:     " << UPort << endl;
#endif
    if (ParseHost(UHost)) {      
      UUrlPath = USpecific.Copy(curPos, USpecific.StrLength());
#ifdef _U_DEBUG
      cout << "url-path: " << UUrlPath << endl;
#endif     
      if (UScheme == "HTTP") Valid = ParseHttpPath(UUrlPath);
      else if (UScheme == "FTP") Valid = ParseFtpPath(UUrlPath);            
      else {
	Valid = 1;
	StrError = "scheme not implemented";
      }
      
    }
  }
}

int CUrl::ParseHttpPath(const CString& iUrlPath){
  int qPos = iUrlPath.Pos('?');
  if (qPos >= 0) {
    UHttpPath = iUrlPath.Copy(0, qPos);
    UHttpSearchPath = iUrlPath.Copy(qPos+1, iUrlPath.StrLength());
    int sPos = UHttpSearchPath.Pos('#');
    if (sPos >= 0) {
      UHttpArg = UHttpSearchPath.Copy(0, sPos);
      UHttpSig = UHttpSearchPath.Copy(sPos+1, UHttpSearchPath.StrLength());
    } else UHttpArg = UHttpSearchPath;
  } else UHttpPath = iUrlPath;
#ifdef _U_DEBUG
  cout << "h-path:   " << UHttpPath << endl;
  cout << "h-search: " << UHttpSearchPath << endl;
  cout << "h-arg:    " << UHttpArg << endl;
  cout << "h-sig:    " << UHttpSig << endl;
#endif
  return 1;
}

int CUrl::ParseFtpPath(const CString& iUrlPath){
  int curPos = 0;  
  int prevPos = 0;
  UFtpPathType = 0;
  while (curPos < iUrlPath.StrLength()) {
    switch(iUrlPath[curPos]){
    case '/':
#ifdef _U_DEBUG
      cout << "ftp-item: " << iUrlPath.Copy(prevPos, curPos-prevPos) << endl;
#endif
      UFtpPath+=iUrlPath.Copy(prevPos, curPos-prevPos);
      prevPos = curPos+1;
      break;
    case ';':
#ifdef _U_DEBUG
      cout << "ftp-item: " << iUrlPath.Copy(prevPos, curPos-prevPos) << endl;
#endif
      UFtpPath+=iUrlPath.Copy(prevPos, curPos-prevPos);
      prevPos = curPos+1;
      if (iUrlPath.Copy(prevPos, strlen("type=")) == "type=") {
	curPos+=(strlen("type=")+1);
	UFtpPathType = iUrlPath[curPos];
#ifdef _U_DEBUG
	cout << "ftp-type: " << UFtpPathType << endl;
#endif
	if (iUrlPath.StrLength() > (curPos+1)){
	  StrError = "FTP type too long";
	  return 0;
	}	
	return 1;
      } else {
	StrError = "invalid FTP type";
	return 0;
      }
    }
    curPos++;
  }
  return 1;
}

int CUrl::ParseUrlSpecific(int& curPos, const CString& iUrl){
  curPos = 0;
  int prevPos = 0;
  CString curVector;
  while (curPos < iUrl.StrLength()) {
    switch(iUrl[curPos]){
    case ':':
      if (curVector.StrLength()) {
	StrError = "invalid host/username";
	return 0;
      }
      curVector=iUrl.Copy(prevPos, curPos-prevPos);
      prevPos=curPos+1;
      break;
    case '@':
      if (curVector.StrLength()) {
	UUser = curVector;
	UPassword = iUrl.Copy(prevPos, curPos-prevPos);
	curVector.Free();
      } else UUser = iUrl.Copy(prevPos, curPos-prevPos);
      prevPos = curPos+1;
      break;
    case '/':
      if (curVector.StrLength()) {
	UHost = curVector;
	UPort = iUrl.Copy(prevPos, curPos-prevPos);
	if (!UPort.Val()) {
	  StrError = "invalid port";
	  return 0;
	}
	curVector.Free();	
      } else UHost = iUrl.Copy(prevPos, curPos-prevPos);      
      curPos++;
      return 1;
    }
    curPos++;
  }
  if (curVector.StrLength()) {
    UHost = curVector;
    UPort = iUrl.Copy(prevPos, curPos-prevPos);   
  } else UHost = iUrl.Copy(prevPos, curPos-prevPos);
  return 1;
}

int CUrl::isValid(void) const {
  return Valid;
}


char CUrl::X2C(char * what) const {
  char digit;
  digit = (what[0] >= 'A' ? ((what[0] & 0xdf) - 'A')+10 : (what[0] - '0'));
  digit *= 16;
  digit += (what[1] >= 'A' ? ((what[1] & 0xdf) - 'A')+10 : (what[1] - '0'));
  return(digit);
}

char * CUrl::MakeC2X(char c){
  workbuf[0]='%';
  sprintf(workbuf+1, "%02X", c);
  workbuf[3] = 0; 
  return workbuf;
}

CString CUrl::Escape(const CString& Brute) {
  CString Tmp;
  if (Brute.StrLength()) {
    int prev = 0;
    for (int i=0;i<Brute.StrLength();i++){
      if (!(isalnum(Brute[i]))&&(Brute[i]!=' ')&&(Brute[i]>0)){
	if (i>prev) Tmp+=Brute.Copy(prev, i-prev);
	prev = i+1;
	Tmp += MakeC2X(Brute[i]);
      }
    }
    Tmp+=Brute.Copy(prev, Brute.StrLength()-prev);        
  }
  return Tmp;
}

CString CUrl::UnEscape(const CString& Brute) const {
  CString Result;
  if (Brute.StrLength()){
    char * cBrute = strdup(Brute.asString());    
    int x,y;
    for(x=0,y=0;cBrute[y];++x,++y) {
      if(((cBrute[x] = cBrute[y]) == '%')) {
        cBrute[x] = X2C(&cBrute[y+1]);
        y+=2;
      }
    }
    cBrute[x] = 0;
    Result += cBrute;    
    delete cBrute;
  }
  return Result;
}

int CUrl::ParseHost(const CString& iHost){
  int curPos = 0;
  int prevPos = 0;
  while (isalnum(iHost[curPos]) || (iHost[curPos]=='-') || (iHost[curPos]=='.')) {
    switch(iHost[curPos]) {
    case '.':
      UHostVector+=iHost.Copy(prevPos, curPos - prevPos);
      prevPos = curPos+1;
    }
    curPos++;
  }
  if (curPos != iHost.StrLength()-1) {
    StrError = "invalid host";
    return 1;
  } else return 0;
}

int CUrl::operator==(const CUrl& iUrl){  
  return (
	  (UScheme == iUrl.UScheme)&&
	  (UUser == iUrl.UUser)&&
	  (UPassword == iUrl.UPassword)&&
	  (UPort == iUrl.UPort)&&
	  (UHost == iUrl.UHost)&&
	  (UUrlPath == iUrl.UUrlPath)&&
	  (UFtpPathType == iUrl.UFtpPathType)&&
	  (UHttpPath == iUrl.UHttpPath)&&
	  (UHttpArg == iUrl.UHttpArg)&&
	  (UHttpSig == iUrl.UHttpSig)&&
	  (Valid)&&(iUrl.Valid));	  			       
}

CString CUrl::GetBrute(void) const { return UBrute; }
CString CUrl::GetScheme(void) const { return UScheme; }
CString CUrl::GetSpecific(void) const { return USpecific; }
CString CUrl::GetUser(void) const { return UnEscape(UUser); }
CString CUrl::GetPassword(void) const { return UnEscape(UPassword); }
CString CUrl::GetPort(void) const { return UnEscape(UPort); }
int CUrl::GetPortValue(void) const { return UnEscape(UPort).Val(); }
CString CUrl::GetHost(void) const { return UnEscape(UHost); }
CString CUrl::GetUrlPath(void) const { return UnEscape(UUrlPath); }
char CUrl::GetFtpPathType(void) const { return UFtpPathType; }
CVector<CString> CUrl::GetHostVector(void) const { return UHostVector; }
CVector<CString> CUrl::GetFtpPath(void) const { return UFtpPath; }
CString CUrl::GetHttpPath(void) const { return UHttpPath; }
CString CUrl::GetHttpSearchPath(void) const { return UHttpSearchPath; }
CString CUrl::GetHttpArg(void) const { return UHttpArg; }
CString CUrl::GetHttpSig(void) const { return UHttpSig; }


