/*
  CUrl - URL implementation
  =========================
  [RFC1738] 
  "Uniform Resource Locators", T. Berners-Lee, L. Masinter, and M. McCahill, December 1994.
  Available at http://ds.internic.net/rfc/rfc1738.txt. 
  http://www.faqs.org/rfcs/rfc1738.html
  
  (c) Daniel Doubrovkine - 1998 (dblock@vestris.com)

  + implemented full general syntax analysis and particular schemes
  (note: for unimplemented schemes, Valid is set to 1 but StrError says "unimplemented scheme"
  + HTTP
  + FTP  
  */

#ifndef C_URL_H
#define C_URL_H

#include "../cstring/cstring.h"

class CUrl {
private:
  char workbuf[24];
  CString UBrute;
  
  CString UScheme;
  CString USpecific;
  CString UUser;
  CString UPassword;
  CString UPort;
  CString UHost;
  CString UUrlPath;  
  CVector<CString> UHostVector;
  CVector<CString> UFtpPath;
  char UFtpPathType;

  CString UHttpPath;
  CString UHttpSearchPath;
  CString UHttpArg;
  CString UHttpSig;

  int Valid;
  CString ReadScheme(int& curPos, const CString& iUrl);
  CString Escape(const CString&);
  CString UnEscape(const CString&) const;
  char * MakeC2X(char c);
  char X2C(char * what) const;
  void ParseUrl(const CString&);
  int ParseUrlSpecific(int&, const CString&);
  int ParseHost(const CString&);
  CString StrError;
  int ParseFtpPath(const CString&);
  int ParseHttpPath(const CString&);
public:
  CUrl(void);
  CUrl(const CString&);
  void SetUrl(const CString&);
  int operator==(const CUrl&);
  int isValid(void) const;    
  CString GetBrute(void) const;
  CString GetScheme(void) const;
  CString GetSpecific(void) const;
  CString GetUser(void) const;
  CString GetPassword(void) const;
  CString GetPort(void) const;
  int GetPortValue(void) const;
  CString GetHost(void) const;
  CString GetUrlPath(void) const;
  char GetFtpPathType(void) const;
  CVector<CString> GetHostVector(void) const;
  CVector<CString> GetFtpPath(void) const;
  CString GetHttpPath(void) const;
  CString GetHttpSearchPath(void) const;
  CString GetHttpArg(void) const;
  CString GetHttpSig(void) const;
};

#endif
