#include "inetsock.h"
#include <errno.h>
#include "../platform/platform.h"
#include "../platform/cgi_aid.h"

//#define _S_DEBUG

CString wsLastError;

void inetSocket::wserror(const CString& cError){
  wsLastError = Name;
  wsLastError += " - ";
  wsLastError += cError;
  Error = 1;
  return;
}

void inetSocket::expect110_ok(void){
  int i; if ((i = recv(S,R,255,0))< 0) wserror("error waiting for 110 ack"); R[i]='\0';
  strncpy(T, R, 3); T[3]='\0';
  if (strncmp(T, "+OK", 3)!= 0) wserror(R+5);
}

void inetSocket::expect25_ok(int code) {
  int i; if ((i = recv(S,R,255,0))< 0) wserror("error receiving port reply"); R[i]='\0';
  strncpy(T, R, 3); T[3]='\0';
  if ((code >= 0)&&(atoi(T) != code)) wserror(R+3);
}

void inetSocket::send25_ok(const CString& string, int code){
  if (send(S, string.asString(), string.StrLength(), 0)<0) wserror("error sending to port 25");
  if (code >= 0) expect25_ok(code);
}

void inetSocket::send110_ok(const CString& string){
  if (send(S,string.asString(), string.StrLength(), 0)<0) wserror("error sending to port 110");
  expect110_ok();
}

inetSocket::inetSocket(int cPort, const CString& cName) {
  Port = cPort;
  Name = cName;
  S = 0;
#ifndef ANSI
  WSADATA W;
  WSAStartup (0x101, &W);
#endif
  Reopen();  
}

void inetSocket::Reopen(void){
#ifdef _S_DEBUG
  cout << "inetSocket::Reopen()" << endl;
#endif
  if (S) {
    shutdown(S, 1);
    close(S);
  }
  Error = 0;  
  wsLastError.Free();
#ifdef _S_DEBUG
  cout << "inetSocket::Reopen() - wsLastError reset" << endl;
#endif
  if ((S = socket(PF_INET, SOCK_STREAM, 0)) == -1) {
#ifdef _S_DEBUG
    cout << "inetSocket::Reopen() - socket error (" << S << ")" << endl;
#endif
    Error = 1;
    wsLastError = Name;
    switch(errno){
    case EACCES: wsLastError+=" - The  process  does   not   have   appropriate privileges."; break;
#ifdef ANSI
    case EAFNOSUPPORT: wsLastError+=" - The  implementation  does  not  support   the specified address family."; break;
    case EPROTOTYPE: wsLastError+=" - The socket type is not supported by the  protocol."; break;
    case ENOBUFS: wsLastError+=" - Insufficient resources were available in  the system to perform the operation."; break;
    case ENOSR: wsLastError+=" - There  were  insufficient  STREAMS  resources available for the operation to complete."; break;   
    case EPROTONOSUPPORT: wsLastError+=" - The protocol is not supported by the  address family,  or  the protocol is not supported by the implementation."; break;
#endif
    case EMFILE: wsLastError+=" - No more file descriptors  are  available  for this process."; break;
    case ENFILE: wsLastError+=" - No more file descriptors  are  available  for the system."; break;    
    case ENOMEM: wsLastError+=" - Insufficient memory was available to  fulfill the request."; break;    
    default: wsLastError+=" - Unable to create socket, unexpected error."; break;
    }
    return;
  }
#ifdef _S_DEBUG
  cout << "inetSocket::Reopen() - socket created successfully" << endl;
#endif
  A.sin_family=PF_INET;
  A.sin_port = htons(Port);
  unsigned long addr;
  if ((int)(addr = inet_addr(Name.asString())) != -1) {   
    A.sin_addr.s_addr = inet_addr(Name.asString());
  } else {
    H = gethostbyname(Name.asString());
    if(!H) {            
      Error = 1;
      wsLastError = Name;
      switch(h_errno){
      case HOST_NOT_FOUND: wsLastError+=" - No such host is known."; break;
      case TRY_AGAIN: wsLastError+=" - A  temporary  and  possibly  transient  error occurred,  such  as  a failure of a server to respond."; break;
      case NO_RECOVERY: wsLastError+=" - An unexpected server failure  occurred  which can not be recovered."; break;
      case NO_DATA: wsLastError+=" - The server recognized  the  request  and  the name  but  no  address is available.  Another type of request to the name  server  for  the domain might return an answer."; break;
      default: wsLastError+=" - gethostbyname() has failed, unexpected error."; break;
      }
      return;
    }
    A.sin_addr.s_addr=*((unsigned long *) H->h_addr);    
  }
#ifdef _S_DEBUG
  cout << "inetSocket::Reopen() - attempting to connect" << endl;
#endif
  if (connect(S,(struct sockaddr *) &A, sizeof(A)) == -1) {    
    Error = 1;
    wsLastError = Name;
    switch(errno){
#ifdef ANSI
    case EADDRNOTAVAIL: wsLastError+=" - The specified address is not  available  from the local machine."; break;
    case EAFNOSUPPORT: wsLastError+=" - The specified address is not a valid address for the address family of the specified socket."; break;
    case EALREADY: wsLastError+=" - A connection request is already  in  progress for the specified socket."; break;
    case ECONNREFUSED: wsLastError+=" - The target address was not listening for connections or refused the connection request."; break;
    case EINPROGRESS: wsLastError+=" - O_NONBLOCK is set for the file descriptor for the  socket  and  the  connection  cannot  be immediately established; the connection  will be established asynchronously."; break;
    case EISCONN: wsLastError+=" - The specified socket is  connection-mode  and is already connected."; break;
    case ENETUNREACH: wsLastError+=" - No route to the network is present."; break;
    case ENOTSOCK: wsLastError+=" - The socket  argument  does  not  refer  to  a socket."; break;
    case EPROTOTYPE: wsLastError+=" - The specified address has  a  different  type than  the  socket bound to the specified peer address."; break;
    case ETIMEDOUT: wsLastError+=" - The attempt to connect  timed  out  before  a connection was made."; break;
    case ELOOP: wsLastError+=" - Too many symbolic links were  encountered  in translating the pathname in address."; break;
    case EADDRINUSE: wsLastError+=" - Attempt to establish a connection  that  uses addresses that are already in use."; break;
    case ECONNRESET: wsLastError+=" - Remote host reset the connection request."; break;
    case EHOSTUNREACH: wsLastError+=" - The destination host cannot be reached (probably  because  the  host  is down or a remote router cannot reach it)."; break;
    case ENETDOWN: wsLastError+=" - The local interface used to reach the  destination is down."; break;
    case ENOBUFS: wsLastError+=" - No buffer space is available."; break;
    case ENOSR: wsLastError+=" - There  were  insufficient  STREAMS  resources available to complete the operation."; break;
    case EOPNOTSUPP: wsLastError+=" - The socket is listening and can not  be  conected."; break;	    
#endif
    case EBADF: wsLastError+=" - The socket  argument  is  not  a  valid  file descriptor."; break;        
    case EINTR: wsLastError+=" - The attempt to  establish  a  connection  was interrupted  by delivery of a signal that was caught; the connection  will  be  established asynchronously."; break;        
    case ENOTDIR: wsLastError+=" - A component of the path prefix of  the  pathname in address is not a directory."; break;
      //case ENAMETOOLONG: wsLastError+=" - A component of a pathname  exceeded  NAME_MAX characters,  or  an  entire pathname exceeded PATH_MAX characters."; break;
    case EACCES: wsLastError+=" - Search permission is denied for  a  component of  the  path  prefix; or write access to the named socket is denied."; break;
    case EIO: wsLastError+=" - An I/O error occurred while reading  from  or writing to the file system."; break;
    case ENOENT: wsLastError+=" - A component of the pathname does not name  an existing  file  or  the  pathname is an empty string."; break;
    case EINVAL: wsLastError+=" - The  address_len  argument  is  not  a  valid length  for  the  address  family; or invalid address family in sockaddr structure."; break;
    case ENAMETOOLONG: wsLastError+=" - Pathname resolution of a symbolic  link  produced  an  intermediate  result  whose length exceeds PATH_MAX."; break;    
    default: wsLastError+=" - connect() has failed, unexpected error "; wsLastError+=errno; wsLastError+="."; break;
    }
  }
  if(wsLastError.StrLength()) return;
  if (Port == 110) expect110_ok(); 
  else if (Port == 25) expect25_ok(220);
}

inetSocket::~inetSocket(void){
  shutdown(S, 1);
  close(S);
}
