/*
  management host
  */

#include "MHost.h"
#include "../cgiout/cgiout.h"
#include "../html_form/html_classes.h"
#include "../nextgen/nextgen_v2.h"
#include "../inetsock/email.h"
#include "../certif/certif.h"
#include "../cmail/cmail.h"

#define M_SIZE "40"
#define M_TWIDTH "100%"
#define TD1WIDTH "25%"
#define EUndefinedRoot "unable to determine"

MHost::MHost(cgiOutStream& CGIStream, int mType){
  Type = mType;  
  CGIStream << "<html>\n<head>\n<title>" << HostId() << " scripting host</title>\n</head>\n<body bgcolor=\"white\">\n";
}

CString MHost::HostId(void){
  if (Type == M_AGNES) return "aGNeS News Forum";
  else if (Type == M_GUEST) return "GuestBook CGI";
  else if (Type == M_FORUM) return "Simple Forum CGI";
  else if (Type == M_LINKS) return "Links Launcher CGI";
  else if (Type == M_IRMA) return "IRMA Forms CGI";
  else if (Type == M_ASEARCH) return "Alkaline Search Engine";
  else return "unsupported scripting host type";
}

void MHost::StrOp(cgiOutStream& CGIStream, const CString& Str) const {
  CGIStream << Str << " ... \n"; CGIStream.flush(); 
}

void MHost::StrOp(cgiOutStream& CGIStream, const CString& StrC, const CString& StrV) const {
  CString Tmp(StrC);  
  Tmp+=" [<b>"; Tmp+=StrV; Tmp+="</b>]";
  StrOp(CGIStream, Tmp);
}

void MHost::StrOk(cgiOutStream& CGIStream, const CString& Str) const {
  CGIStream << "<font color=\"blue\">" << Str << "!</font>" << elf;
}

void MHost::StrOk(cgiOutStream& CGIStream) const {
  StrOk(CGIStream, "ok");
}

void MHost::StrError(cgiOutStream& CGIStream, const CString& Str) const {
  CGIStream << "<font color=\"red\">" << Str << "!</font>" << elf;
}

void MHost::StrError(cgiOutStream& CGIStream) const {
  StrError(CGIStream, "invalid");
}

int MHost::CheckAdminPassword(cgiOutStream& CGIStream, pParam_v2& Params, cgi_equiv_manager& Equivs){
  StrOp(CGIStream, "checking administrative password");
  CString Password = Access.get_value(CGIStream, "manage-add",0);
  if (Password.StrLength() && (Params.get_entry("PASSWORD")!=Password)) {
    StrError(CGIStream);
    return 0;
  } else StrOk(CGIStream);
  return 1;
}

void MHost::ProcessPost(cgiOutStream& CGIStream, pParam_v2& Params, cgi_equiv_manager& Equivs){
  CString Action = Params.get_entry("ACTION");
  if (Action == "AGNES-ADD") ProcessPostAgnes(CGIStream, Params, Equivs);
  else if (Action == "GUEST-ADD") ProcessPostGuest(CGIStream, Params, Equivs);
  else if (Action == "FORUM-ADD") ProcessPostForum(CGIStream, Params, Equivs);
  else if (Action == "LINKS-ADD") ProcessPostLinks(CGIStream, Params, Equivs);  
  else if (Action == "CERTIFGEN") ProcesPostCertif(CGIStream, Params, Equivs);  
  /*
    globaladmin entry point
    */
  else if ((Type != M_IRMA)&&(Type != M_ASEARCH)) {
    if (Action == "GAGENERAL") GAPostGeneral(CGIStream, Params, Equivs);
    else if (Action == "GAENTRY") GAPostEntry(CGIStream, Params, Equivs);
    else if (Action == "GADELETE") GAPostDelete(CGIStream, Params, Equivs);
    else if (Action == "GACACHE") GAPostCache(CGIStream, Params, Equivs);
    else if (Action == "GAEMAIL") GAPostEmail(CGIStream, Params, Equivs);
    else if (Action == "GATEMPLATE") GAPostTemplate(CGIStream, Params, Equivs);    
    else if (Action == "PASSWD-CHANGE") ProcessPostPasswd(CGIStream, Params, Equivs);
  } else StrError(CGIStream, "action not supported by this scripting host");
  CGIStream << hr_;
}

void MHost::ProcessPostLinks(cgiOutStream& CGIStream, pParam_v2& Params, cgi_equiv_manager& Equivs){
  CVector<CString> Required;
  Required+="<!--MOZILLA-FORM-->";
  Required+="<!--MOZILLA-LIST-->";
  Required+="<!--MOZILLA-->";
  ProcessPostOther(CGIStream, Params,Equivs,"Links",Required);
}

void MHost::ProcessPostPasswd(cgiOutStream& CGIStream, pParam_v2& Params, cgi_equiv_manager& Equivs){
  StrOp(CGIStream, "checking for a selected script alias");
  CString Victim(Params.get_entry("VICTIM"));
  if (!(Victim.StrLength())) {
    StrError(CGIStream, "none chosen - please select your script alias");
    return;
  } else StrOk(CGIStream);   
  CString OldPass(Params.get_entry("PASS-OLD"));  
  StrOp(CGIStream, "checking administrative password");

  if ((Access.get_value(CGIStream, Victim, 0) != OldPass)||(!OldPass.StrLength())) {
    StrError(CGIStream);
    StrOp(CGIStream, "checking root password");    
    if ((Access.get_value(CGIStream, "root", 0) != OldPass)||(!OldPass.StrLength())) {
      StrError(CGIStream);
      return;
    } else StrOk(CGIStream);
  } else StrOk(CGIStream);   
  CString NewPass(Params.get_entry("PASS-NEW"));
  StrOp(CGIStream, "checking new password", "*****");
  if ((NewPass.StrLength() < 4)||(NewPass.isSubStr(' ') != -1)) {
    StrError(CGIStream, "password too short (must be at least 4 chars) or contains spaces");
    return;
  } else StrOk(CGIStream);

  StrOp(CGIStream, "checking if confirmation password matches");  
  if (NewPass != Params.get_entry("PASS-NEW2")) {
    StrError(CGIStream);
    return;
  } else StrOk(CGIStream);  
  Access.set_value(Victim, NewPass);
  StrOp(CGIStream, "flushing data to access.struct");
  if (Access.write()) StrOk(CGIStream); else StrError(CGIStream);
}

void MHost::ProcessPostGuest(cgiOutStream& CGIStream, pParam_v2& Params, cgi_equiv_manager& Equivs){
  CVector<CString> Required;
  Required+="<!--MOZILLA-LIST-->";
  Required+="<!--MOZILLA-->";
  Required+="<!--MOZILLA-FORM-->";
  Required+="<!--MOZILLA-BOOK-->";
  ProcessPostOther(CGIStream, Params, Equivs, "Guest", Required);
}

void MHost::ProcessPostForum(cgiOutStream& CGIStream, pParam_v2& Params, cgi_equiv_manager& Equivs){
  CVector<CString> Required;
  Required+="<!--MOZILLA-->";
  Required+="<!--MOZILLA-LIST-->";
  ProcessPostOther(CGIStream, Params, Equivs, "Forum", Required);
}


void MHost::ProcessPostOther(cgiOutStream& CGIStream, 
			     pParam_v2& Params, cgi_equiv_manager& Equivs, 
			     const CString& PAlias,			     
			     const CVector<CString>& Required){

  if (!CheckAdminPassword(CGIStream, Params, Equivs)) return;    
  
  CString LAlias(PAlias); LAlias.LowerCase();
  CString NA_Name = Params.get_entry("NAME");
  CString NA_Email = Params.get_entry("EMAIL");
  CString NA_Alias = Params.get_entry("ALIAS");
  CString NA_Url = Params.get_entry("URL");
  CString NA_Pass = Params.get_entry("PASS");
  CString NA_Pass2 = Params.get_entry("PASS2");

  int Error = 0;
  
  StrOp(CGIStream, "checking name", NA_Name); 
  if (!NA_Name.StrLength()) {StrError(CGIStream, "no name specified");Error = 1;} else StrOk(CGIStream);
  
  StrOp(CGIStream, "checking e-mail", NA_Email);
  CMail Mail(NA_Email);
  if ((!NA_Email.StrLength())||(!Mail.isGoodInternet())) {
    StrError(CGIStream, "no email or invalid email specified");
    Error = 1;
  } else StrOk(CGIStream);
  
  StrOp(CGIStream, "checking alias for valid length", NA_Alias);    
  if ((NA_Alias.StrLength() > 10)||(NA_Alias.StrLength() < 3)) {
    StrError(CGIStream, "alias too long or too short, must be [3-10] characters");
    Error = 1;
  } else StrOk(CGIStream);
  
  CString t(NA_Alias); t.escape_url(); 
  StrOp(CGIStream, "checking alias for valid syntax", NA_Alias);    
  if ((!NA_Alias.StrLength())||(t != NA_Alias)||(NA_Alias.isSubStr(' ')!=-1)) {
    StrError(CGIStream, "no alias or invalid alias specified, must be single word and with no special characters");
    Error = 1;
  } else StrOk(CGIStream);
  
  StrOp(CGIStream, "checking if alias already exists", NA_Alias);
  t = NA_Alias; t+=PAlias;
  if (Equivs.get_value(CGIStream, t, 0).StrLength()) {
    StrError(CGIStream, "sorry, this alias is already used, pick a different one");
    Error = 1;
  } else StrOk(CGIStream, "it does not, good!");
 
  StrOp(CGIStream, "checking if URL is correct", NA_Url);
  if (!(NA_Url.StartsWith("http://"))) {
    StrError(CGIStream, "invalid url, must be http://...");
    Error = 1;
  } else StrOk(CGIStream);
  
  StrOp(CGIStream, "checking password", "*****");
  if ((NA_Pass.StrLength() < 4)||(NA_Pass.isSubStr(' ') != -1)||(NA_Pass!=NA_Pass2)) {
    StrError(CGIStream, "password too short (must be at least 4 chars), contains spaces or confirmed password does not match");
    Error = 1;
  } else StrOk(CGIStream);
  
  if (Error) return;    
  
#ifdef proxy
  CGIStream << "removing proxy setting ...";
  Proxy.Free();
#endif
  
  StrOp(CGIStream, "checking url existence, this may take a while", NA_Url);
  nextgen_v2 gen(NA_Url); 
  if (gen.void_read) {
    CString tmp = "invalid url or invalid file name<br>error returned:"; tmp+=gen.contained;
    StrError(CGIStream, tmp);      
    return;
  } else {
    CString tmp = "read "; tmp+= gen.contained.StrLength(); tmp+=" bytes, good";
    StrOk(CGIStream, tmp);
  }
    
  StrOp(CGIStream, "checking template content", NA_Url);
  
  Error = 1;
  for (int j=0;j<Required.Count();j++)    
    if (gen.contained.isSubStr(Required[j]) != -1) {
      Error = 0;
      break;
    }
  if (Error) {
    StrError(CGIStream, "invalid template file, must contain at least a valid tag - this means that your template page url is false or page is not accessible");
    return;
  } else StrOk(CGIStream, "ok, found a valid tag");

  CGIStream << "<b>HAL will be happy to add your new script, please be patient ...</b>" << elf;    
  CGIStream << "adding contact definition:<br><b># " << NA_Name << " &lt;" << NA_Email << "&gt;</b>" << elf;
  CGIStream << "adding equiv structure for contents:" << elf;
  CVector<CString> FirstSetNames;
  CVector<CString> FirstSetValues;
  
  NA_Alias.LowerCase();
  NA_Alias[0]-=('a'-'A');
  
  FirstSetNames+=NA_Alias;
  FirstSetNames+=PAlias;    
  t = getcwd(0,0);t.append_backslash();t+="data";t.append_backslash(); t+=LAlias; t.append_backslash();
  CString root(t); CString t2(NA_Alias.LowerCase()); t2+=".";t2+=LAlias;t+=t2; 
  FILE * F = fopen(t.asString(), "r");
  if (F) {
    StrError(CGIStream, "sorry, alias used from a different structure, please pick a different one");
    fclose(F);
    return;
  }

  fclose(fopen(t.asString(), "w+"));
   
  FirstSetValues+=root;
  FirstSetValues+=t2;  
  
  CGIStream << b_;    
  int i;
  for (i=0;i<FirstSetNames.Count();i++) { if (i) CGIStream << "+"; CGIStream << FirstSetNames[i];}
  CGIStream << ",";
  for (i=0;i<FirstSetValues.Count();i++) { if (i) CGIStream << "+"; CGIStream << FirstSetValues[i];}
  CGIStream << !b_ << elf;
  
  StrOp(CGIStream, "creating equiv structure for templates");
  CVector<CString> SecondSetNames;
  CVector<CString> SecondSetValues;
  NA_Alias[0]-=('a'-'A');
  t = NA_Alias; t+=PAlias; t+="HTML";
  SecondSetNames+=t;  
  SecondSetValues+=NA_Url;
  
  CGIStream << b_;
  for (i=0;i<SecondSetNames.Count();i++) { if (i) CGIStream << "+"; CGIStream << SecondSetNames[i];}
  CGIStream << ",";
  for (i=0;i<SecondSetValues.Count();i++) { if (i) CGIStream << "+"; CGIStream << SecondSetValues[i];}
  CGIStream << !b_ << elf;
  CGIStream << hr_ << elf;

  CString tCmd = getenv("SCRIPT_NAME"); tCmd+="?manage";
  CGIStream << b_ << a_.o("HREF", tCmd.asString(), 0) << "BOOKMARK THIS ADMINISTRATION URL" << !a_ << !b_ << hr_ << elf;
  tCmd = getenv("SCRIPT_NAME");  
  tCmd+="?"; tCmd+=NA_Alias; tCmd+=PAlias;tCmd+="+"; tCmd+=NA_Alias; tCmd+=PAlias;tCmd+="HTML";
  CGIStream << font_.o("SIZE","+1",0) << b_ << a_.o("HREF", tCmd.asString(), 0) << "CLICK HERE FOR YOUR NEW SCRIPT!" << !b_ << !font_
	 << !a_ << br_ << "the url is: " << b_ << "http://" << getenv("SERVER_NAME") << tCmd << !b_ << hr_ << elf;
  
  CString iStat("/home/dblock/stat/"); iStat+=LAlias; iStat+=".stat";
  nextgen_v2 stat(iStat);
  CGIStream << font_.o("SIZE","-1", 0) << stat.contained << !font_ << elf;
  CGIStream << hr_ << elf;
  
  CEmail SEmail;
  CString RootEmail(Equivs.get_value(CGIStream, "ROOT-EMAIL", 0)); if (!RootEmail.StrLength()) RootEmail = "dblock@vestris.com";
  SEmail.from = RootEmail;		
  SEmail.subject = "Welcome to the interactive world! ("; SEmail.subject+=PAlias; SEmail.subject+=")";
  SEmail.name = "Script Admin";
  SEmail.contents = getenv("SERVER_NAME");
  SEmail.contents+= tCmd;
  SEmail.contents+="\n\n";
  SEmail.contents+=stat.contained.remove_br();    
  SEmail.to = NA_Email;
  CString MServer(Equivs.get_value(CGIStream, "MailServer", 0)); if (!MServer.StrLength()) MServer = "mail.vestris.com";
  SEmail.Send(MServer);
  
  NA_Alias+=PAlias;
  Access.write_full_access(NA_Alias, NA_Pass);
  Admin.write_full_admin(LAlias,NA_Alias);
  Equivs.write_full_equiv(NA_Name, NA_Email, FirstSetNames, FirstSetValues, SecondSetNames, SecondSetValues); 
}

void MHost::ProcessPostAgnes(cgiOutStream& CGIStream, pParam_v2& Params, cgi_equiv_manager& Equivs){  
  if (!CheckAdminPassword(CGIStream, Params, Equivs)) return;    
  
  CString NA_Name = Params.get_entry("NAME");
  CString NA_Email = Params.get_entry("EMAIL");
  CString NA_Alias = Params.get_entry("ALIAS");
  CString NA_Url = Params.get_entry("URL");
  CString NA_Pass = Params.get_entry("PASS");
  CString NA_Pass2 = Params.get_entry("PASS2");
  CString NA_HeaderHTML = Params.get_entry("header.html");
  CString NA_ArticleHTML = Params.get_entry("article.html");
  CString NA_FollowHTML = Params.get_entry("follow.html");
  CString NA_PostNewHTML = Params.get_entry("postnew.html");
  CString NA_SearchHTML = Params.get_entry("search.html");
  CString NA_PreviewHTML = Params.get_entry("preview.html");
  
  int Error = 0;
  
  StrOp(CGIStream, "checking name", NA_Name); 
  if (!NA_Name.StrLength()) {StrError(CGIStream, "no name specified");Error = 1;} else StrOk(CGIStream);
  
  StrOp(CGIStream, "checking e-mail", NA_Email);
  CMail Mail(NA_Email);
  if ((!NA_Email.StrLength())||(!Mail.isGoodInternet())) {
    StrError(CGIStream, "no email or invalid email specified");
    Error = 1;
  } else StrOk(CGIStream);
  
  StrOp(CGIStream, "checking alias for valid length", NA_Alias);    
  if ((NA_Alias.StrLength() > 10)||(NA_Alias.StrLength() < 3)) {
    StrError(CGIStream, "alias too long or too short, must be [3-10] characters");
    Error = 1;
  } else StrOk(CGIStream);
  
  CString t(NA_Alias); t.escape_url(); 
  StrOp(CGIStream, "checking alias for valid syntax", NA_Alias);    
  if ((!NA_Alias.StrLength())||(t != NA_Alias)||(NA_Alias.isSubStr(' ')!=-1)) {
    StrError(CGIStream, "no alias or invalid alias specified, must be single word and with no special characters");
    Error = 1;
  } else StrOk(CGIStream);
  
  StrOp(CGIStream, "checking if alias already exists", NA_Alias);
  t = NA_Alias; t+="Agnes";
  if (Equivs.get_value(CGIStream, t, 0).StrLength()) {
    StrError(CGIStream, "sorry, this alias is already used, pick a different one");
    Error = 1;
  } else StrOk(CGIStream, "it does not, good!");
  
  StrOp(CGIStream, "checking if file names are empty");
  if ((!NA_HeaderHTML.StrLength())||(!NA_FollowHTML.StrLength())||(!NA_PostNewHTML.StrLength())||(!NA_SearchHTML.StrLength())||(!NA_PreviewHTML.StrLength())) {
    StrError(CGIStream, "one of the header.html/article.html/... files is empty");
    Error = 1;
  } else StrOk(CGIStream);
  
  StrOp(CGIStream, "checking if URL is correct", NA_Url);
  if (!(NA_Url.StartsWith("http://"))) {
    StrError(CGIStream, "invalid url, must be http://...");
    Error = 1;
  } else StrOk(CGIStream);
  
  StrOp(CGIStream, "checking password", "*****");
  if ((NA_Pass.StrLength() < 4)||(NA_Pass.isSubStr(' ') != -1)||(NA_Pass!=NA_Pass2)) {
    StrError(CGIStream, "password too short (must be at least 4 chars), contains spaces or confirmed password does not match");
    Error = 1;
  } else StrOk(CGIStream);
  
  if (Error) return;    
  
#ifdef proxy
  CGIStream << "removing proxy setting ...";
  Proxy.Free();
#endif
  
  NA_Url.append_backslash();
  t = NA_Url; t += NA_HeaderHTML;
  StrOp(CGIStream, "checking header.html existence, this may take a while", t);
  nextgen_v2 gen(t); 
  if (gen.void_read) {
    CString tmp = "invalid url or invalid file name<br>error returned:"; tmp+=gen.contained;
    StrError(CGIStream, tmp);      
    return;
  } else {
    CString tmp = "read "; tmp+= gen.contained.StrLength(); tmp+=" bytes, good";
    StrOk(CGIStream, tmp);
  }
    
  StrOp(CGIStream, "checking header.html content", t);
  if (gen.contained.isSubStr("<!--AGNES-HEADER-->") == -1) {
    StrError(CGIStream, "invalid header.html file, must contain at least a valid aGNeS tag -  - this means that your header.html url is false or page is not accessible");
    return;
  } else StrOk(CGIStream, "ok, found a valid aGNeS tag, assuming other files are okay");
  
  CGIStream << "<b>HAL will be happy to add your new aGNeS forum, please be patient ...</b>" << elf;    
  CGIStream << "adding contact definition:<br><b># " << NA_Name << " &lt;" << NA_Email << "&gt;</b>" << elf;
  CGIStream << "adding equiv structure for contents:" << elf;
  CVector<CString> FirstSetNames; 
  CVector<CString> FirstSetValues;
  
  NA_Alias.LowerCase();
  NA_Alias[0]-=('a'-'A');
  
  t = NA_Alias; t+="Agnes";
  FirstSetNames+=t;
  FirstSetNames+="Parent";
  FirstSetNames+="";
  t = getcwd(0,0);t.append_backslash();t+="data";t.append_backslash(); t+="agnes"; t.append_backslash();
  CString root(t); t+=NA_Alias.LowerCase();
  if (!chdir(t.asString())) {
    StrError(CGIStream, "sorry, alias used from a different structure, please pick a different one");
    return;
  } 
  StrOp(CGIStream, "creating structure directory", t);
#ifdef ANSI
  mkdir(t.asString(), S_IRWXU);
#else
  mkdir(t.asString());
#endif
  if (chdir(t.asString())) {
    StrError(CGIStream, "error creating directory");
    return;
  }
  t.append_backslash();
#ifdef ANSI
  CString t2 = "cp -R "; t2+=root; t2+="template/* "; t2+=t;
  StrOp(CGIStream, "executing", t2);    
  system(t2.asString());
#endif
    
  FirstSetValues+=t;
  FirstSetValues+="";
  FirstSetValues+="header.list";
  
  CGIStream << b_;    
  int i;
  for (i=0;i<FirstSetNames.Count();i++) { if (i) CGIStream << "+"; CGIStream << FirstSetNames[i];}
  CGIStream << ",";
  for (i=0;i<FirstSetValues.Count();i++) { if (i) CGIStream << "+"; CGIStream << FirstSetValues[i];}
  CGIStream << !b_ << elf;
  
  StrOp(CGIStream, "creating equiv structure for templates");
  CVector<CString> SecondSetNames;
  CVector<CString> SecondSetValues;
  NA_Alias[0]-=('a'-'A');
  t = NA_Alias; t+="Agnes"; t+="HTML";
  SecondSetNames+=t;
  SecondSetNames+="";
  SecondSetNames+="Search";
  SecondSetNames+="Followup";
  SecondSetNames+="PostNew";
  SecondSetNames+="Article";
  SecondSetNames+="Preview";
  SecondSetValues+=NA_Url;
  SecondSetValues+=NA_HeaderHTML;
  SecondSetValues+=NA_SearchHTML;
  SecondSetValues+=NA_FollowHTML;
  SecondSetValues+=NA_PostNewHTML;
  SecondSetValues+=NA_ArticleHTML;
  SecondSetValues+=NA_PreviewHTML;
  
  CGIStream << b_;
  for (i=0;i<SecondSetNames.Count();i++) { if (i) CGIStream << "+"; CGIStream << SecondSetNames[i];}
  CGIStream << ",";
  for (i=0;i<SecondSetValues.Count();i++) { if (i) CGIStream << "+"; CGIStream << SecondSetValues[i];}
  CGIStream << !b_ << elf;
  CGIStream << hr_ << elf;
  CString tCmd = getenv("SCRIPT_NAME"); tCmd+="?manage";
  CGIStream << b_ << a_.o("HREF", tCmd.asString(), 0) << "BOOKMARK THIS ADMINISTRATION URL" << !a_ << !b_ << hr_ << elf;  
  tCmd = getenv("SCRIPT_NAME"); 
  tCmd+="?"; tCmd+=NA_Alias; tCmd+="Agnes+"; tCmd+=NA_Alias; tCmd+="AgnesHTML";
  CGIStream << font_.o("SIZE","+1",0) << b_ << a_.o("HREF", tCmd.asString(), 0) << "CLICK HERE FOR YOUR NEW FORUM!" << !b_ << !font_
	    << !a_ << br_ << "the url is: " << b_ << "http://" << getenv("SERVER_NAME") << tCmd << !b_ << hr_ << elf;
  
  nextgen_v2 stat("/home/dblock/stat/agnes.stat");
  CGIStream << font_.o("SIZE","-1", 0) << stat.contained << !font_ << elf;
  CGIStream << hr_ << elf;
  
  CEmail SEmail;
  CString RootEmail(Equivs.get_value(CGIStream, "ROOT-EMAIL", 0)); if (!RootEmail.StrLength()) RootEmail = "dblock@vestris.com";
  SEmail.from = RootEmail;	
  SEmail.subject = "Welcome to the interactive world!";
  SEmail.name = "aGNeS Admin";
  SEmail.contents = "http://www.vestris.com";
  SEmail.contents+= tCmd;
  SEmail.contents+="\n\n";
  SEmail.contents+=stat.contained.remove_br();    
  SEmail.to = NA_Email;
  CString MServer(Equivs.get_value(CGIStream, "MailServer", 0)); if (!MServer.StrLength()) MServer = "mail.vestris.com";
  SEmail.Send(MServer);

  NA_Alias+="Agnes";
  Access.write_full_access(NA_Alias, NA_Pass);    
  Admin.write_full_admin("agnes",NA_Alias);
  Equivs.write_full_equiv(NA_Name, NA_Email, FirstSetNames, FirstSetValues, SecondSetNames, SecondSetValues); 
}

void MHost::PasswordInput(cgiOutStream& CGIStream, pParam_v2& Params, cgi_equiv_manager& Equivs){
  if (Access.get_value(CGIStream, "manage-add",0).StrLength()) 
    CGIStream << tr_ << td_.o("WIDTH",TD1WIDTH,"ALIGN","RIGHT", 0) << "Admin Password: " << !td_ << td_ << input_.o("NAME","PASSWORD","TYPE","PASSWORD","SIZE",M_SIZE,0) << !td_ << !tr_ << "\n";
}

void MHost::AliasSelect(cgiOutStream& CGIStream, pParam_v2& Params, cgi_equiv_manager& Equivs){
  CGIStream << select_.o("NAME","VICTIM",0) << "\n";
  CVector<CString> SVector;
  for (int i=0;i<Equivs.EAlias.entries_count();i++) SVector+=Equivs.EAlias.get_name(i);
  SVector.QSort();
  for (int j=0;j<SVector.Count();j++)
    CGIStream << option_.o("VALUE",SVector[j].asString(), "", "SELECTED", 0) << SVector[j] << !option_ << "\n";
  //CGIStream << input_.o("TYPE","RADIO","NAME","VICTIM","VALUE",Equivs.EAlias.get_name(i).asString(), 0) << " " << Equivs.EAlias.get_name(i) << !input_ << elf;
  CGIStream << !select_;
}

void MHost::PassChangeForm(cgiOutStream& CGIStream, pParam_v2& Params, cgi_equiv_manager& Equivs){
  Equivs.make_oce();
  
  CGIStream << form_.o("ACTION",Cmd.asString(),"METHOD","POST",0) << "\n";
  CGIStream << table_.o("WIDTH",M_TWIDTH, 0); 
  CGIStream << tr_ << td_.o("WIDTH",TD1WIDTH,"ALIGN","RIGHT", 0) << "Choose your data alias:" << !td_ << td_ << "\n";
  AliasSelect(CGIStream, Params, Equivs);
  CGIStream << !td_ << !tr_ << "\n";    
  CGIStream << input_.o("NAME","ACTION","TYPE","HIDDEN","VALUE","PASSWD-CHANGE",0) << "\n";
  CGIStream << tr_ << td_.o("WIDTH",TD1WIDTH,"ALIGN","RIGHT", 0) << "Current Password: " << !td_ << td_ << input_.o("NAME","PASS-OLD","TYPE","PASSWORD","SIZE",M_SIZE,0) << !td_ << !tr_ << "\n";
  CGIStream << tr_ << td_.o("WIDTH",TD1WIDTH,"ALIGN","RIGHT", 0) << "New Password:" << !td_ << td_ << input_.o("NAME","PASS-NEW","TYPE","PASSWORD","SIZE",M_SIZE, 0) << !td_ << !tr_ << "\n";
  CGIStream << tr_ << td_.o("WIDTH",TD1WIDTH,"ALIGN","RIGHT", 0) << "Confirm New Password:" << !td_ << td_ << input_.o("NAME","PASS-NEW2","TYPE","PASSWORD","SIZE",M_SIZE, 0) << !td_ << !tr_ << "\n";
  CGIStream << tr_ << td_.o("WIDTH",TD1WIDTH,"ALIGN","RIGHT", 0) << !td_ << td_ << input_.o("NAME","SUBMIT","TYPE","SUBMIT","VALUE","Modify!", 0) << !td_ << !tr_ << "\n";
  CGIStream << !table_ << !form_ << "\n";
}

void MHost::OtherAddForm(cgiOutStream& CGIStream, pParam_v2& Params, cgi_equiv_manager& Equivs, const CString& DefAction, const CString& DefUrl){
  CGIStream << form_.o("ACTION",Cmd.asString(),"METHOD","POST",0) << "\n";
  CGIStream << table_.o("WIDTH",M_TWIDTH, 0); 
  PasswordInput(CGIStream, Params, Equivs);
  CGIStream << input_.o("NAME","ACTION","TYPE","HIDDEN","VALUE",DefAction.asString(),0) << "\n";
  CGIStream << tr_ << td_.o("WIDTH",TD1WIDTH,"ALIGN","RIGHT", 0) << "Contact Name: " << !td_ << td_ << input_.o("NAME","NAME","TYPE","TEXT","SIZE",M_SIZE,0) << !td_ << !tr_ << "\n";
  CGIStream << tr_ << td_.o("WIDTH",TD1WIDTH,"ALIGN","RIGHT", 0) << "Contact E-Mail: " << !td_ << td_ << input_.o("NAME","EMAIL","TYPE","TEXT","SIZE",M_SIZE,0) << !td_ << !tr_ << "\n";
  CGIStream << tr_ << td_.o("WIDTH",TD1WIDTH,"ALIGN","RIGHT", 0) << "Alias:<br><font size=\"-1\">(any simple single short word<br>that describes your site best)</font>" << !td_ << td_ << input_.o("NAME","ALIAS","TYPE","TEXT","SIZE",M_SIZE,0) << !td_ << !tr_ << "\n";
  CGIStream << tr_ << td_.o("WIDTH",TD1WIDTH,"ALIGN","RIGHT", 0) << "URL of the template file:" << !td_ << td_ << input_.o("NAME","URL","TYPE","TEXT","VALUE",DefUrl.asString(),"SIZE",M_SIZE, 0) << !td_ << !tr_ << "\n";
  CGIStream << tr_ << td_.o("WIDTH",TD1WIDTH,"ALIGN","RIGHT", 0) << "Password:<br><font size=\"-1\">(used for entries maintaince,<br>please remember it)</font>" << !td_ << td_ << input_.o("NAME","PASS","TYPE","PASSWORD","SIZE",M_SIZE, 0) << !td_ << !tr_ << "\n";
  CGIStream << tr_ << td_.o("WIDTH",TD1WIDTH,"ALIGN","RIGHT", 0) << "Confirm Password:" << !td_ << td_ << input_.o("NAME","PASS2","TYPE","PASSWORD","SIZE",M_SIZE, 0) << !td_ << !tr_ << "\n";
  CGIStream << tr_ << td_.o("WIDTH",TD1WIDTH,"ALIGN","RIGHT", 0) << !td_ << td_ << input_.o("NAME","SUBMIT","TYPE","SUBMIT","VALUE","Confirm!", 0) << !td_ << !tr_ << "\n";
  CGIStream << tr_ << td_.o("WIDTH",TD1WIDTH,"ALIGN","RIGHT", 0) << !td_ << td_ << "submission might take up to a minute,<br>please be patient" << !td_ << !tr_ << "\n";
  CGIStream << !table_ << !form_ << "\n";
}

void MHost::AgnesAddForm(cgiOutStream& CGIStream, pParam_v2& Params, cgi_equiv_manager& Equivs){
  CGIStream << form_.o("ACTION",Cmd.asString(),"METHOD","POST",0) << "\n";
   CGIStream << table_.o("WIDTH",M_TWIDTH, 0); 
  PasswordInput(CGIStream, Params, Equivs);
  CGIStream << input_.o("NAME","ACTION","TYPE","HIDDEN","VALUE","AGNES-ADD",0) << "\n";
  CGIStream << tr_ << td_.o("WIDTH",TD1WIDTH,"ALIGN","RIGHT", 0) << "Contact Name: " << !td_ << td_ << input_.o("NAME","NAME","TYPE","TEXT","SIZE",M_SIZE,0) << !td_ << !tr_ << "\n";
  CGIStream << tr_ << td_.o("WIDTH",TD1WIDTH,"ALIGN","RIGHT", 0) << "Contact E-Mail: " << !td_ << td_ << input_.o("NAME","EMAIL","TYPE","TEXT","SIZE",M_SIZE,0) << !td_ << !tr_ << "\n";
  CGIStream << tr_ << td_.o("WIDTH",TD1WIDTH,"ALIGN","RIGHT", 0) << "aGNeS Alias:<br><font size=\"-1\">(any simple single short word<br>that describes your site best)</font>" << !td_ << td_ << input_.o("NAME","ALIAS","TYPE","TEXT","SIZE",M_SIZE,0) << !td_ << !tr_ << "\n";
  CGIStream << tr_ << td_.o("WIDTH",TD1WIDTH,"ALIGN","RIGHT", 0) << "URL of the demo pages:<br><font size=\"-1\">(without any file names)</font>" << !td_ << td_ << input_.o("NAME","URL","TYPE","TEXT","VALUE","http://yoursite.url/agnes/","SIZE",M_SIZE, 0) << !td_ << !tr_ << "\n";

  CGIStream << tr_ << td_.o("WIDTH",TD1WIDTH,"ALIGN","RIGHT", 0) << "File Names:<br><font size=\"-1\">(without the urls,<br>adjust in case you have<br>an .htm extension<br>or different names)</font>" << !td_ << td_	 
	 << input_.o("NAME","header.html","TYPE","TEXT","VALUE","header.html","SIZE",M_SIZE, 0) << br_ << "\n"
	 << input_.o("NAME","article.html","TYPE","TEXT","VALUE","article.html","SIZE",M_SIZE, 0) << br_ << "\n"
	 << input_.o("NAME","postnew.html","TYPE","TEXT","VALUE","postnew.html","SIZE",M_SIZE, 0) << br_ << "\n"
	 << input_.o("NAME","follow.html","TYPE","TEXT","VALUE","follow.html","SIZE",M_SIZE, 0) << br_ << "\n"
	 << input_.o("NAME","preview.html","TYPE","TEXT","VALUE","preview.html","SIZE",M_SIZE, 0) << br_ << "\n"
	 << input_.o("NAME","search.html","TYPE","TEXT","VALUE","search.html","SIZE",M_SIZE, 0)	<< "\n"
	 << !td_ << !tr_ << "\n";
 
  CGIStream << tr_ << td_.o("WIDTH",TD1WIDTH,"ALIGN","RIGHT", 0) << "aGNeS Forum Password:<br><font size=\"-1\">(used for forum maintaince,<br>please remember it)</font>" << !td_ << td_ << input_.o("NAME","PASS","TYPE","PASSWORD","SIZE",M_SIZE, 0) << !td_ << !tr_ << "\n";
  CGIStream << tr_ << td_.o("WIDTH",TD1WIDTH,"ALIGN","RIGHT", 0) << "Confirm Password:" << !td_ << td_ << input_.o("NAME","PASS2","TYPE","PASSWORD","SIZE",M_SIZE, 0) << !td_ << !tr_ << "\n";
  
  CGIStream << tr_ << td_.o("WIDTH",TD1WIDTH,"ALIGN","RIGHT", 0) << !td_ << td_ << input_.o("NAME","SUBMIT","TYPE","SUBMIT","VALUE","Confirm!", 0) << !td_ << !tr_ << "\n";
  CGIStream << tr_ << td_.o("WIDTH",TD1WIDTH,"ALIGN","RIGHT", 0) << !td_ << td_ << "submission might take up to a minute,<br>please be patient" << !td_ << !tr_ << "\n";

  
  CGIStream << !table_ << !form_ << "\n";
}

void MHost::ProcessGet(cgiOutStream& CGIStream, pParam_v2& Params, cgi_equiv_manager& Equivs){  
  if (Access.get_value(CGIStream, "root", 0).StrLength() || (Type == M_IRMA) || (Type == M_ASEARCH)) {	
    CString tCmd;
    switch(Type){
    case M_IRMA:
    case M_ASEARCH:
      break;
    case M_FORUM:
      if ((Params.argv.Count() >= 3)&&(Params.argv[2] == "add")) OtherAddForm(CGIStream, Params, Equivs, "FORUM-ADD", "http://www.yoursite.url/forum/forum.html");
      tCmd = Cmd; tCmd+="+add";
      CGIStream << "[ " << a_.o("HREF", tCmd.asString(), 0) << "create a new Simple Forum" << !a_ << " ]" << elf;    
      break;
    case M_GUEST:
      if ((Params.argv.Count() >= 3)&&(Params.argv[2] == "add")) OtherAddForm(CGIStream, Params, Equivs, "GUEST-ADD", "http://www.yoursite.url/guest/guest.html");
      tCmd = Cmd; tCmd+="+add";
      CGIStream << "[ " << a_.o("HREF", tCmd.asString(), 0) << "create a new GuestBook" << !a_ << " ]" << elf;    
      break;
    case M_AGNES:
      if ((Params.argv.Count() >= 3)&&(Params.argv[2] == "add")) AgnesAddForm(CGIStream, Params, Equivs);    
      tCmd = Cmd; tCmd+="+add";
      CGIStream << "[ " << a_.o("HREF", tCmd.asString(), 0) << "create a new aGNeS forum" << !a_ << " ]" << elf;    
      break;
    case M_LINKS:
      if ((Params.argv.Count() >= 3)&&(Params.argv[2] == "add")) OtherAddForm(CGIStream, Params, Equivs, "LINKS-ADD", "http://www.yourside.url/links/links.html");
      tCmd = Cmd; tCmd+="+add";
      CGIStream << "[ " << a_.o("HREF", tCmd.asString(), 0) << "create a new Links Launcher" << !a_ << " ]" << elf;    
      break;
    }	
    tCmd = Cmd; tCmd+="+general";
    CGIStream << "[ " << a_.o("HREF", tCmd.asString(), 0) << "this group administration" << !a_ << " ]" << elf;
    tCmd = Cmd; tCmd+="+list";
    CGIStream << "[ " << a_.o("HREF", tCmd.asString(), 0) << "list all equivalence structures" << !a_ << " ]" << elf;
    tCmd = Cmd; tCmd+="+obvious";
    CGIStream << "[ " << a_.o("HREF", tCmd.asString(), 0) << "list all obvious associations" << !a_ << " ]" << elf;
    tCmd.StrDelete(tCmd.InvPos('/')+1, tCmd.StrLength()); tCmd+="cache_update?CacheUpdateHTML";
    CGIStream << "[ " << a_.o("HREF", tCmd.asString(), 0) << "group cache updater (obsolete)" << !a_ << " ]" << elf;
    tCmd = Cmd; tCmd+="+admin";  
    CGIStream << "[ " << a_.o("HREF", tCmd.asString(), 0) << "forum/guestbook/links administration" << !a_ << " ]" << elf;    
    CGIStream << "[ " << a_.o("HREF", "http://www.vestris.com/agnes/split.html", 0) << "Vestris Inc. forum for questions" << !a_ << " ]" << elf;
    if (Params.argv.Count() >= 3) {
      if (Params.argv[2] == "list") Equivs.output_equivs(CGIStream);
      else if (Params.argv[2] == "general") GAGeneral(CGIStream, Params, Equivs, 0);
      else if (Params.argv[2] == "certif") ProcessCertif(CGIStream, Params, Equivs);	  
      else if (Params.argv[2] == "obvious") {
	CGIStream << hr_ << "constructing obvious associations:" << hr_;
	Equivs.make_obvious();
	for(int k=0;k<Equivs.EObvious.Count();k++) CGIStream << Equivs.EObvious[k] << elf;
	CGIStream << hr_ << "(this group) agnes: " << Equivs.l_agnes << " guest: " << Equivs.l_guest << " forum: " << Equivs.l_forum << " links: " << Equivs.l_links << elf;  
      } else if (Params.argv[2] == "pass") PassChangeForm(CGIStream, Params, Equivs);
      else if (Params.argv[2] == "admin") GAEntry(CGIStream, Params, Equivs);
    } else {
      GAEntry(CGIStream, Params, Equivs);
      Equivs.make_obvious();
      CGIStream << hr_ << "(this group) agnes: " << Equivs.l_agnes << " guest: " << Equivs.l_guest << " forum: " << Equivs.l_forum << " links: " << Equivs.l_links << elf;  
    }
  } else if ((Type != M_IRMA)&&(Type != M_ASEARCH)) GAGeneral(CGIStream, Params, Equivs, 1);
}

void MHost::Operate(cgiOutStream& CGIStream, pParam_v2& Params, cgi_equiv_manager& Equivs){
  Cmd = getenv("SCRIPT_NAME"); 
  if (Params.argv.Count() >= 2) {
    Cmd+="?"; Cmd+=Params.argv[1];
  }
  CGIStream << blockquote_ << "scripting host type: " << b_ << HostId() << !b_  << " - (" << __DATE__ << ")" << elf;
  CString MV4Group(getenv("SCRIPT_NAME")); 
  int sPos = MV4Group.InvPos('/');
  if (sPos >= 0) {
    MV4Group.StrDelete(sPos, MV4Group.StrLength());
    sPos = MV4Group.InvPos('/');
    if (sPos >= 0) {
      MV4Group.StrInsert(sPos+1,b_.o("","",0));
      MV4Group+=!b_;
    }
    CGIStream << "group: " << MV4Group << elf;
  }
  
  CString Banner = Equivs.get_value(CGIStream, "Banner", 0); if (Banner.StrLength()) 
    CGIStream << br_ << a_.o("HREF", "http://www.vestris.com", 0) << img_.o("SRC",Banner.asString(),0) << !a_ << br_ << elf;
  
  if (Params.isPost() == 1) ProcessPost(CGIStream, Params, Equivs);  
  ProcessGet(CGIStream, Params, Equivs);
  
  CString RootEmail(Equivs.get_value(CGIStream, "Root-Email", 0)); if (!RootEmail.StrLength()) RootEmail = "dblock@vestris.com";
  CString RootEmailLink("mailto:"); RootEmailLink+=RootEmail; RootEmailLink+="?Subject=Question from MV4 Scripting Host";
  CGIStream << hr_ << "(this is an automatic script process, for all questions or problems, please e-mail " << a_.o("HREF",RootEmailLink.asString(),0) << RootEmail << !a_ << ")" << elf; 
  CGIStream << !blockquote_ << "</body>\n</html>";
}

void MHost::GAEntry(cgiOutStream& CGIStream, pParam_v2& Params, cgi_equiv_manager& Equivs){
  if (Access.get_value(CGIStream, "root", 0).StrLength()) {
    Equivs.make_oce();  
    CGIStream << form_.o("ACTION",Cmd.asString(),"METHOD","POST",0) << "\n";
    CGIStream << table_.o("WIDTH",M_TWIDTH, 0); 
    CGIStream << tr_ << td_.o("WIDTH",TD1WIDTH,"ALIGN","RIGHT", 0) << "Choose your data alias:" 
	   << br_ << font_.o("SIZE","-2",0) << "(after login you can" << br_ << "change admin password, delete an alias" << br_ << "update cache, change URLs and" << br_ << "modify admin parameters)" << !font_ << !td_ << td_ << "\n";
    AliasSelect(CGIStream, Params, Equivs);
    CGIStream << !td_ << !tr_ << "\n";    
    CGIStream << input_.o("NAME","ACTION","TYPE","HIDDEN","VALUE","GAENTRY",0) << "\n";
    CGIStream << tr_ << td_.o("WIDTH",TD1WIDTH,"ALIGN","RIGHT", 0) << "Password: " << !td_ << td_ << input_.o("NAME","PASSWORD","TYPE","PASSWORD",0) << !td_ << !tr_ << "\n";
    CGIStream << tr_ << td_.o("WIDTH",TD1WIDTH,"ALIGN","RIGHT", 0) << !td_ << td_ << input_.o("NAME","SUBMIT","TYPE","SUBMIT","VALUE","MV4 Administration", 0) << !td_ << !tr_ << "\n";
    CGIStream << !table_ << !form_ << "\n";
  } else GAGeneral(CGIStream, Params, Equivs, 1);
}

void MHost::GAPostDelete(cgiOutStream& CGIStream, pParam_v2& Params, cgi_equiv_manager& Equivs){
  CString Victim;
  if (GAPostCommon(CGIStream, Params, Equivs, Victim)) {    
    StrOp(CGIStream, "deleting admin alias");
    Admin.entry_manager::del_value(Victim);
    Admin.entry_manager::set_value(Victim, "deleted", Victim);            
    StrOk(CGIStream);    
    StrOp(CGIStream, "flushing data to admin.struct");
    if (Admin.write()) StrOk(CGIStream); else StrError(CGIStream);     
  }
}

void MHost::GAPostGeneral(cgiOutStream& CGIStream, pParam_v2& Params, cgi_equiv_manager& Equivs){
	CString NewPassword(Params.get_entry("NEWPASSWORD"));
	CString OldPassword(Params.get_entry("PASSWORD"));
	CString RootPassword(Access.get_value(CGIStream, "root", 0));
	StrOp(CGIStream, "checking for root password existance");
	if (RootPassword.StrLength()) {	
		StrOk(CGIStream);
		StrOp(CGIStream, "checking root password (entire group administrator)");    
		if (RootPassword != OldPassword) {
			StrError(CGIStream);
			return;
		} else StrOk(CGIStream);
	} else StrError(CGIStream, "none");
	
	StrOp(CGIStream, "checking for a new password");
	if (NewPassword.StrLength()){
		StrOk(CGIStream);
		StrOp(CGIStream, "checking new password", "*****");
		if ((NewPassword.StrLength() < 4)||(NewPassword.isSubStr(' ') != -1)) {
			StrError(CGIStream, "password too short (must be at least 4 chars) or contains spaces");
			return;
		} else StrOk(CGIStream);

		StrOp(CGIStream, "checking if confirmation password matches");  
		if (NewPassword != Params.get_entry("NEWPASSWORD2")) {
			StrError(CGIStream);
			return;
		} else StrOk(CGIStream);
	
		Access.set_value("root", NewPassword);
		StrOp(CGIStream, "flushing data to access.struct");
		if (Access.write()) StrOk(CGIStream); else StrError(CGIStream);	
	} else StrError(CGIStream, "ignoring");

	CString NewEmail(Params.get_entry("EMAIL"));	
	CString RootEmail(Equivs.get_value(CGIStream, "Root-Email", 0)); 
	StrOp(CGIStream, "checking for new admin email");
	if (NewEmail.StrLength()) {
		if (NewEmail != RootEmail){
		 if ((!NewEmail.StrLength())||(NewEmail.isSubStr('@') == -1)||(NewEmail.isSubStr('.') == -1)||(NewEmail.isSubStr(' ')!=-1)) {
			StrError(CGIStream, "invalid email specified");
			return;
		 } else {
			 StrOk(CGIStream);
			 StrOp(CGIStream, "flushing equiv.struct");
			 if (Equivs.replace_equiv("Root-Email",NewEmail,1)) StrOk(CGIStream); else StrError(CGIStream);
		 }
		} else StrError(CGIStream, "ignoring");
	} else StrOp(CGIStream, "ignoring");

	CString NewBlank(Params.get_entry("BLANK"));	
	CString OldBlank(Equivs.get_value(CGIStream, "BLANK", 0)); 
	StrOp(CGIStream, "checking for new blank url");
	if (NewBlank.StrLength()) {
		if (NewBlank != OldBlank){
		 StrOk(CGIStream);
		 StrOp(CGIStream, "flushing equiv.struct");
			if (Equivs.replace_equiv("Blank",NewBlank,1)) StrOk(CGIStream); else StrError(CGIStream);
		} else StrError(CGIStream, "ignoring");
	} else StrOp(CGIStream, "ignoring");

	CString NewMailServer(Params.get_entry("MAILSERVER"));	
	CString OldMailServer(Equivs.get_value(CGIStream, "MailServer", 0)); 
	StrOp(CGIStream, "checking for new mail server");
	if (NewMailServer.StrLength()) {
		if (NewMailServer != OldMailServer){
		 StrOk(CGIStream);
		 StrOp(CGIStream, "flushing equiv.struct");
			if (Equivs.replace_equiv("MailServer",NewMailServer,1)) StrOk(CGIStream); else StrError(CGIStream);
		} else StrError(CGIStream, "ignoring");
	} else StrOp(CGIStream, "ignoring");

	CString NewManageAdd(Params.get_entry("MANAGEADD"));	
	CString OldManageAdd(Equivs.get_value(CGIStream, "manage-add", 0)); 
	StrOp(CGIStream, "checking for new management password (forum creation)");
	if (NewManageAdd.StrLength()) {
	  if (NewManageAdd != OldManageAdd){
	    StrOk(CGIStream);
	    StrOp(CGIStream, "flushing equiv.struct");
	    if (Equivs.replace_equiv("Manage-add",NewManageAdd,1)) StrOk(CGIStream); else StrError(CGIStream);
	  } else StrError(CGIStream, "ignoring");
	} else StrOp(CGIStream, "ignoring");
}

int MHost::GAPostCommon(cgiOutStream& CGIStream, pParam_v2& Params, cgi_equiv_manager& Equivs, CString& Victim){
  StrOp(CGIStream, "checking for a selected data alias");
  Victim = Params.get_entry("VICTIM");
  if (!(Victim.StrLength())) {
    StrError(CGIStream, "none chosen - please select your script alias");
    return 0;
  } else StrOk(CGIStream);
  CString Password(Params.get_entry("PASSWORD"));  
  StrOp(CGIStream, "checking administrative password (current alias administrator)");
  if ((Access.get_value(CGIStream, Victim, 0) != Password)||(!Password.StrLength())) {
    StrError(CGIStream);
    StrOp(CGIStream, "checking root password (entire group administrator)");    
    if ((Access.get_value(CGIStream, "root", 0) != Password)||(!Password.StrLength())) {
      StrError(CGIStream);
      CString RootEmail(Equivs.get_value(CGIStream, "Root-Email", 0)); if (!RootEmail.StrLength()) RootEmail = "dblock@vestris.com";
      CString EmailAdmin("mailto:"); EmailAdmin+=RootEmail; EmailAdmin+="?Subject=Password for "; EmailAdmin+=Victim; 
      EmailAdmin+=" - "; EmailAdmin+=getenv("SCRIPT_NAME");
      CGIStream << b_ << "forgot your password? " << a_.o("HREF",EmailAdmin.asString(),0) << "e-mail the administrator" << !a_ << elf;
      return 0;
    } else StrOk(CGIStream);
  } else StrOk(CGIStream);
  StrOp(CGIStream, "primary access granted, calculating magic number"); StrOk(CGIStream);
  return 1;
}

CString MHost::GAMakeRoot(cgiOutStream& CGIStream, const CString& Victim, cgi_equiv_manager& Equivs){
  CString EVictimHTML(Victim); EVictimHTML+="HTML";    
  CString ERoot(Equivs.get_value(CGIStream, EVictimHTML, 0));
  if (ERoot.StrLength()) {
    int sPos = ERoot.InvPos('/'); if (sPos >= 0) ERoot.StrDelete(sPos+1, ERoot.StrLength());
    return ERoot;
  } else return EUndefinedRoot;
}

void MHost::GAPostEntry(cgiOutStream& CGIStream, pParam_v2& Params, cgi_equiv_manager& Equivs){
  CString Victim;
  if (GAPostCommon(CGIStream, Params, Equivs, Victim)) {
    CGIStream << h3_ << "alias: " << b_ << Victim << !b_ << !h3_ << "\n";
    CString ERoot(GAMakeRoot(CGIStream, Victim, Equivs));
    if (ERoot != EUndefinedRoot) CGIStream << "equiv root: " << a_.o("HREF",ERoot.asString(),0) << ERoot << !a_ << elf;
    Equivs.make_asocs();    
    Equivs.make_obvious();    
   
    CString EAlias =  Equivs.EAssocs.get_value("aliased", Victim);
    if (EAlias.StrLength()) {
      CGIStream << a_.o("HREF", EAlias.asString(), 0) << EAlias << !a_ << elf;
      
      if (Type == M_AGNES) {
	CString EAliasTmp;      
	CGIStream << "<ul>";
	CGIStream << "<li>[ "; EAliasTmp = EAlias; EAliasTmp+="+delete"; CGIStream << a_.o("HREF", EAliasTmp.asString(), 0) << "delete articles and archives" << !a_ << "]\n";
	CGIStream << "<li>[ "; EAliasTmp = EAlias; EAliasTmp+="+archive"; CGIStream << a_.o("HREF", EAliasTmp.asString(), 0) << "archive articles" << !a_ << "]\n";
	CGIStream << "<li>[ "; EAliasTmp = EAlias; EAliasTmp+="+edit"; CGIStream << a_.o("HREF", EAliasTmp.asString(), 0) << "edit/moderate articles" << !a_ << "]\n";	
	CGIStream << "</ul>";
      }
    }
    
    GACache(CGIStream, Params, Equivs);   
    GADelete(CGIStream, Params, Equivs);   
    GAPasswd(CGIStream, Params, Equivs);
    GAEmail(CGIStream, Params, Equivs, Victim);
    GATemplate(CGIStream, Params, Equivs, Victim);
  }
}

void MHost::GAPostEmail(cgiOutStream& CGIStream, pParam_v2& Params, cgi_equiv_manager& Equivs){
  CString Victim;
  if (GAPostCommon(CGIStream, Params, Equivs, Victim)) {
    CString NA_Email = Params.get_entry("ADMIN-EMAIL");
    CString NA_Name = Params.get_entry("ADMIN-NAME");
    StrOp(CGIStream, "checking e-mail", NA_Email);
    CMail Mail(NA_Email);
    if ((!NA_Email.StrLength())||(!Mail.isGoodInternet())) {
      StrError(CGIStream, "no email or invalid email specified");
      return;
    } else StrOk(CGIStream);
    StrOp(CGIStream, "checking name", NA_Name); 
    if (!NA_Name.StrLength()) {
      StrError(CGIStream, "no name specified");
      return;
    } else StrOk(CGIStream);
    StrOp(CGIStream, "writing name and email to equiv.struct");
    StrOp(CGIStream, Victim);
    if (!Equivs.write_email(NA_Name, NA_Email, Victim)) {
      StrError(CGIStream, "error, please contact the administrator");
    } else StrOk(CGIStream);    
  }  
}

void MHost::GAPostTemplate(cgiOutStream& CGIStream, pParam_v2& Params, cgi_equiv_manager& Equivs){
  CString Victim;
  if (GAPostCommon(CGIStream, Params, Equivs, Victim)) {
    CString Template = Params.get_entry("TEMPLATE");    
    StrOp(CGIStream, "checking if URL is correct", Template);
    if (!(Template.StartsWith("http://"))) {
      StrError(CGIStream, "invalid url, must be http://...");
      return;
    } else StrOk(CGIStream);    
    StrOp(CGIStream, "writing name and email to equiv.struct");
    if (!Equivs.write_template(Template, Victim, GAMakeRoot(CGIStream, Victim, Equivs))) {
      StrError(CGIStream, "error, please contact the administrator");
    } else StrOk(CGIStream);    
  }  
}

void MHost::GAPostCache(cgiOutStream& CGIStream, pParam_v2& Params, cgi_equiv_manager& Equivs){
  StrOp(CGIStream, "checking for a selected data alias");
  CString Victim = Params.get_entry("VICTIM");
  if (!(Victim.StrLength())) {
    StrError(CGIStream, "none chosen - please select your script alias");
    return;
  } else StrOk(CGIStream);

  CString cache_equiv = Equivs.get_value(CGIStream, "Cache", 0);
  if (cache_equiv.StrLength() == 0) cgi_error(CGIStream, "this server does not cache html retrievals");
  for (int i=0; i< Params.entries_count();i++)
    if (Params.get_name(i) == "ALIAS") {
      CString ce = Equivs.get_value(CGIStream, Params.get_entry(i), 0);
      if (ce.StrLength()) {	
	CGIStream << "updating cache for " << b_ << Params.get_entry(i) << !b_ << elf;
#ifdef _DEBUG
	debug++;
#endif
	nextgen_v2 dummy(ce, cache_equiv, 0);
#ifdef _DEBUG
	debug--;
#endif
      } else CGIStream << "no equivalence for " << b_ << ce << !b_ << elf;
      CGIStream.flush();
    }
  StrOp(CGIStream, "synching cache progress mechanism"); StrOk(CGIStream);
}

void MHost::GAPasswd(cgiOutStream& CGIStream, pParam_v2& Params, cgi_equiv_manager& Equivs){
  CGIStream << hr_ << b_ << "3. modify administrator password:" << !b_ << "\n";
  CGIStream << form_.o("ACTION",Cmd.asString(),"METHOD","POST",0) << "\n";
  CGIStream << table_.o("WIDTH",M_TWIDTH, 0); 
  CGIStream << input_.o("NAME","VICTIM","TYPE","HIDDEN","VALUE",Params.get_entry("VICTIM").asString(),0) << "\n";
  CGIStream << input_.o("NAME","ACTION","TYPE","HIDDEN","VALUE","PASSWD-CHANGE",0) << "\n";
  CGIStream << tr_ << td_.o("WIDTH",TD1WIDTH,"ALIGN","RIGHT", 0) << "Current Password: " << !td_ << td_ << input_.o("NAME","PASS-OLD","TYPE","PASSWORD","SIZE",M_SIZE,0) << !td_ << !tr_ << "\n";
  CGIStream << tr_ << td_.o("WIDTH",TD1WIDTH,"ALIGN","RIGHT", 0) << "New Password:" << !td_ << td_ << input_.o("NAME","PASS-NEW","TYPE","PASSWORD","SIZE",M_SIZE, 0) << !td_ << !tr_ << "\n";
  CGIStream << tr_ << td_.o("WIDTH",TD1WIDTH,"ALIGN","RIGHT", 0) << "Confirm New Password:" << !td_ << td_ << input_.o("NAME","PASS-NEW2","TYPE","PASSWORD","SIZE",M_SIZE, 0) << !td_ << !tr_ << "\n";
  CGIStream << tr_ << td_.o("WIDTH",TD1WIDTH,"ALIGN","RIGHT", 0) << !td_ << td_ << input_.o("NAME","SUBMIT","TYPE","SUBMIT","VALUE","Modify Password", 0) << !td_ << !tr_ << "\n";
  CGIStream << !table_ << !form_ << "\n";  
}

void MHost::GADelete(cgiOutStream& CGIStream, pParam_v2& Params, cgi_equiv_manager& Equivs){
  CGIStream << hr_ << b_ << "2. delete/destroy alias (this cannot be undone):" << !b_ << "\n";
  CGIStream << form_.o("ACTION",Cmd.asString(),"METHOD","POST",0) << "\n";
  CGIStream << table_.o("WIDTH",M_TWIDTH, 0); 
  CGIStream << input_.o("NAME","VICTIM","TYPE","HIDDEN","VALUE",Params.get_entry("VICTIM").asString(),0) << "\n";
  CGIStream << input_.o("NAME","ACTION","TYPE","HIDDEN","VALUE","GADELETE",0) << "\n";
  CGIStream << tr_ << td_.o("WIDTH",TD1WIDTH,"ALIGN","RIGHT", 0) << "Password: " << !td_ << td_ << input_.o("NAME","PASSWORD","TYPE","PASSWORD","SIZE",M_SIZE,0) << !td_ << !tr_ << "\n";
  CGIStream << tr_ << td_.o("WIDTH",TD1WIDTH,"ALIGN","RIGHT", 0) << !td_ << td_ << input_.o("NAME","SUBMIT","TYPE","SUBMIT","VALUE","Delete Alias", 0) << !td_ << !tr_ << "\n";
  CGIStream << !table_ << !form_ << "\n";
}

void MHost::GATemplate(cgiOutStream& CGIStream, pParam_v2& Params, cgi_equiv_manager& Equivs, const CString& Victim){
  CGIStream << hr_ << b_ << "5. modify the URL of the template pages:" << !b_ << "\n";
  CGIStream << form_.o("ACTION",Cmd.asString(),"METHOD","POST",0) << "\n";
  CGIStream << table_.o("WIDTH",M_TWIDTH, 0); 
  CGIStream << input_.o("NAME","VICTIM","TYPE","HIDDEN","VALUE",Params.get_entry("VICTIM").asString(),0) << "\n";
  CGIStream << input_.o("NAME","ACTION","TYPE","HIDDEN","VALUE","GATEMPLATE",0) << "\n";
  CGIStream << tr_ << td_.o("WIDTH",TD1WIDTH,"ALIGN","RIGHT", 0) << "Password: " << !td_ << td_ << input_.o("NAME","PASSWORD","TYPE","PASSWORD","SIZE",M_SIZE,0) << !td_ << !tr_ << "\n";
  
  CGIStream << tr_ << td_.o("WIDTH",TD1WIDTH,"ALIGN","RIGHT", 0) << "Global Location: " << !td_ << td_ << input_.o("NAME","TEMPLATE","TYPE","TEXT","SIZE",M_SIZE,"VALUE", GAMakeRoot(CGIStream, Victim, Equivs).asString(), 0) << !td_ << !tr_ << "\n";
  
  CGIStream << tr_ << td_.o("WIDTH",TD1WIDTH,"ALIGN","RIGHT", 0) << "resolved:" << !td_ << td_;
  CString Tmp; CString HTMLVictim(Victim); HTMLVictim+="HTML";
  for (int i=0;i<Equivs.entries_count();i++) {
    Tmp = Equivs.get_name(i);
    if (Tmp.StartsWith(HTMLVictim)) CGIStream << Tmp << " = " << b_ << Equivs.get_value(CGIStream, Tmp) << !b_ << elf;
  }
  CGIStream << !td_ << !tr_ << "\n";

  CGIStream << tr_ << td_.o("WIDTH",TD1WIDTH,"ALIGN","RIGHT", 0) << !td_ << td_ << input_.o("NAME","SUBMIT","TYPE","SUBMIT","VALUE","Modify Templates Location", 0) << !td_ << !tr_ << "\n";
  CGIStream << !table_ << !form_ << "\n";
}

void MHost::GAEmail(cgiOutStream& CGIStream, pParam_v2& Params, cgi_equiv_manager& Equivs, const CString& Victim){
  Equivs.make_asocs();
  CString CurEmail(Equivs.EEmail.get_value(Victim)); CurEmail.StrTrim();
  CString CurName;
  int iPos = CurEmail.Pos('<');
  if (iPos > 0) CurName = CurEmail.Copy(0, iPos).StrTrim();
  CurEmail.StrDelete(0, iPos+1);
  iPos = CurEmail.Pos('>');
  if (iPos > 0) CurEmail.StrDelete(iPos, CurEmail.StrLength());
 
  CGIStream << hr_ << b_ << "4. change the administrative name/e-mail:" << !b_ << "\n";
  CGIStream << form_.o("ACTION",Cmd.asString(),"METHOD","POST",0) << "\n";
  CGIStream << table_.o("WIDTH",M_TWIDTH, 0); 
  CGIStream << input_.o("NAME","VICTIM","TYPE","HIDDEN","VALUE",Params.get_entry("VICTIM").asString(),0) << "\n";
  CGIStream << input_.o("NAME","ACTION","TYPE","HIDDEN","VALUE","GAEMAIL",0) << "\n";
  CGIStream << tr_ << td_.o("WIDTH",TD1WIDTH,"ALIGN","RIGHT", 0) << "Password: " << !td_ << td_ << input_.o("NAME","PASSWORD","TYPE","PASSWORD","SIZE",M_SIZE,0) << !td_ << !tr_ << "\n";
  CGIStream << tr_ << td_.o("WIDTH",TD1WIDTH,"ALIGN","RIGHT", 0) << "Name: " << !td_ << td_ << input_.o("NAME","ADMIN-NAME","TYPE","TEXT","SIZE",M_SIZE,"VALUE", CurName.asString(), 0) << !td_ << !tr_ << "\n";  
  CGIStream << tr_ << td_.o("WIDTH",TD1WIDTH,"ALIGN","RIGHT", 0) << "Email: " << !td_ << td_ << input_.o("NAME","ADMIN-EMAIL","TYPE","TEXT","SIZE",M_SIZE,"VALUE", CurEmail.asString(), 0) << !td_ << !tr_ << "\n";  
  CGIStream << tr_ << td_.o("WIDTH",TD1WIDTH,"ALIGN","RIGHT", 0) << !td_ << td_ << input_.o("NAME","SUBMIT","TYPE","SUBMIT","VALUE","Update Name/Email", 0) << !td_ << !tr_ << "\n";
  CGIStream << !table_ << !form_ << "\n";
}

void MHost::GACache(cgiOutStream& CGIStream, pParam_v2& Params, cgi_equiv_manager& Equivs){
  CString Victim(Params.get_entry("VICTIM"));
  CGIStream << hr_ << b_ << "1. cache updater:" << !b_ << "\n";
  CGIStream << form_.o("ACTION",Cmd.asString(),"METHOD","POST",0) << "\n";
  CGIStream << table_.o("WIDTH",M_TWIDTH, 0); 
  CGIStream << input_.o("NAME","VICTIM","TYPE","HIDDEN","VALUE",Victim.asString(),0) << "\n";
  CGIStream << input_.o("NAME","ACTION","TYPE","HIDDEN","VALUE","GACACHE",0) << "\n";
  CGIStream << tr_ << td_.o("WIDTH",TD1WIDTH,"ALIGN","RIGHT", 0) << "update for:" << !td_ << td_;

  CString cache_equiv = Equivs.get_value(CGIStream, "Cache", 0);
  if (cache_equiv.StrLength() == 0) CGIStream << "this server does not cache html retrievals";
  else {
    CVector<CString> c_equivs;
    int i;
    for (i=0;i<Equivs.entries_count();i++) {
      CString ce = Equivs.entry_manager::get_name(i);
      if ((!ce.isSubStrI(Victim))&&((ce.isSubStrI("HTML") >= 0)||(ce.isSubStrI("IRMA")>=0)))  c_equivs+=ce;
    }
    c_equivs.QSort();
    for (i=0;i<c_equivs.Count();i++){
      CGIStream << input_.o("TYPE","checkbox","NAME", "ALIAS", "VALUE", c_equivs[i].asString(), "", "CHECKED",0);
      CGIStream << c_equivs[i] << font_.o("SIZE","-1",0) << " (" << Equivs.get_value(CGIStream, c_equivs[i], 0) << ")" << !font_;
      CGIStream << !input_ << elf;
    }    
    CGIStream << tr_ << td_.o("WIDTH",TD1WIDTH,"ALIGN","RIGHT", 0) << !td_ << td_ << input_.o("NAME","SUBMIT","TYPE","SUBMIT","VALUE","Update Cache", 0) << !td_ << !tr_ << "\n";
  }
  CGIStream << !table_ << !form_ << "\n";
}

CString MHost::MakeCertifCategory(void){
  CString Category;
  if (Type == M_AGNES) Category = "agnes";
  else if (Type == M_IRMA) Category = "irma";  
  else if (Type == M_ASEARCH) Category = "asearch";
  return Category;
}

void MHost::ProcessCertif(cgiOutStream& CGIStream, pParam_v2& Params, cgi_equiv_manager& Equivs){  
  CCertif Certificate(MakeCertifCategory());
  CString CertifKey = Certificate.CalculateCertif();
  CGIStream << hr_ << h3_ << "Certification Key" << !h3_ << hr_;
  CGIStream << table_.o("WIDTH",M_TWIDTH, 0); 
  CGIStream << tr_ << td_.o("WIDTH",TD1WIDTH,"ALIGN","RIGHT","VALIGN","TOP",0);
  CGIStream << form_;
  CGIStream << "email this key to the " << a_.o("HREF", "mailto:admin@vestris.com", 0) << "Vestris Administrator" << !a_;
  CGIStream << !td_ << td_ << textarea_.o("WRAP","VIRTUAL","ROWS","5","COLS","30",0) << CertifKey << !textarea_;
  CGIStream << br_ << "(your server license unlock key will be calculated and emailed to you;" << br_ 
	 << font_.o("COLOR","RED",0) << "you must have purchased a server license from " 
	 << a_.o("HREF","http://www.vestris.com",0) << "Vestris Inc." << !a_ << " first)" << !font_ << !td_ << !tr_ << !table_;
  CGIStream << !form_;
  CGIStream << hr_ << h3_ << "Certificate Generator" << !h3_ << hr_;
  
  CGIStream << form_.o("ACTION",Cmd.asString(),"METHOD","POST",0) << "\n";
  CGIStream << table_.o("WIDTH",M_TWIDTH, 0); 
  CGIStream << input_.o("NAME","KEY","VALUE",CertifKey.asString(), "TYPE","HIDDEN",0) << "\n";
  CGIStream << tr_ << td_.o("WIDTH",TD1WIDTH,"ALIGN","RIGHT", 0) << "Admin Password: " << !td_ << td_ << input_.o("NAME","PASSWORD","TYPE","PASSWORD","SIZE",M_SIZE,0) << !td_ << !tr_ << "\n";
  CGIStream << tr_ << td_.o("WIDTH",TD1WIDTH,"ALIGN","RIGHT", 0) << "unlock key:" << !td_ << td_;
  CGIStream << textarea_.o("NAME","CERTIFICATE","WRAP","VIRTUAL","ROWS","5","COLS","30",0) << !textarea_ << "\n";
  CGIStream << input_.o("NAME","ACTION","TYPE","HIDDEN","VALUE","CERTIFGEN",0) << "\n";
  CGIStream << !td_ << !tr_ << "\n";
  CGIStream << tr_ << td_.o("WIDTH",TD1WIDTH,"ALIGN","RIGHT", 0) << !td_ << td_ << input_.o("NAME","SUBMIT","TYPE","SUBMIT","VALUE","Generate Certificate", 0) << !td_ << !tr_ << "\n";  
  CGIStream << !table_ << !form_ << "\n";
}

int MHost::ProcesPostCertif(cgiOutStream& CGIStream, pParam_v2& Params, cgi_equiv_manager& Equivs){  
  CString Password = Access.get_value(CGIStream, "root",0);
  if (!Password.StrLength()){
	  CGIStream << font_.o("COLOR","RED",0) << "WARNING! No root password defined, you should set one!" << !font_ << elf;
  } 
  StrOp(CGIStream, "checking administrative password");
  if (Password.StrLength() && (Params.get_entry("PASSWORD")!=Password)) {
    StrError(CGIStream);
    return 0;
  } else StrOk(CGIStream);
  
  CCertif Certificate(MakeCertifCategory());
  StrOp(CGIStream, "validating certificate");
  if (Certificate.WriteCertificate(Params.get_entry("CERTIFICATE"))) {
    StrOk(CGIStream);
    return 1; 
  } else {
    StrError(CGIStream, "invalid key");
    return 0;  
  }
}

void MHost::MakeDirectory(cgiOutStream& CGIStream, const CString& Tmp){
  if (chdir(Tmp.asString())) {
    StrOp(CGIStream, "making "); StrOk(CGIStream, Tmp);
#ifdef ANSI
    mkdir(Tmp.asString(), S_IRWXU);
#else
    mkdir(Tmp.asString());
#endif
  }
}

void MHost::GAGeneral(cgiOutStream& CGIStream, pParam_v2& Params, cgi_equiv_manager& Equivs, int New){	
  if ((Type == M_IRMA)||(Type == M_ASEARCH)) return;
  if (New){
    CString CurDir(getcwd(0,0)); CurDir.append_backslash();
    StrOp(CGIStream, "current physical directory: "); StrOk(CGIStream, CurDir);
    CString Tmp(CurDir); Tmp+="equiv";
    MakeDirectory(CGIStream, Tmp);		
    Tmp = CurDir; Tmp+="data";
    MakeDirectory(CGIStream, Tmp);
    Tmp = CurDir; Tmp+="data"; Tmp.append_backslash();Tmp+="agnes";
    MakeDirectory(CGIStream, Tmp);	 
    Tmp = CurDir; Tmp+="data"; Tmp.append_backslash();Tmp+="guest";
    MakeDirectory(CGIStream, Tmp);
    Tmp = CurDir; Tmp+="data"; Tmp.append_backslash();Tmp+="links";
    MakeDirectory(CGIStream, Tmp);
    Tmp = CurDir; Tmp+="data"; Tmp.append_backslash();Tmp+="forum";
    MakeDirectory(CGIStream, Tmp);	  
  }
  CGIStream << hr_ << b_ << "0. first time general setup:" << !b_ << "\n";
  CGIStream << form_.o("ACTION",Cmd.asString(),"METHOD","POST",0) << "\n";
  CGIStream << table_.o("WIDTH",M_TWIDTH, 0); 
  CGIStream << input_.o("NAME","ACTION","TYPE","HIDDEN","VALUE","GAGENERAL",0) << "\n";
  if (Access.get_value(CGIStream, "root", 0).StrLength()) CGIStream << tr_ << td_.o("WIDTH",TD1WIDTH,"ALIGN","RIGHT", 0) << "old admin password: " << !td_ << td_ << input_.o("NAME","PASSWORD","TYPE","PASSWORD","SIZE",M_SIZE,0) << !td_ << !tr_ << "\n";	
  CGIStream << tr_ << td_.o("WIDTH",TD1WIDTH,"ALIGN","RIGHT", 0) << "new admin password: " << !td_ << td_ << input_.o("NAME","NEWPASSWORD","TYPE","PASSWORD","SIZE",M_SIZE,0) << !td_ << !tr_ << "\n";
  CGIStream << tr_ << td_.o("WIDTH",TD1WIDTH,"ALIGN","RIGHT", 0) << "confirm new password: " << !td_ << td_ << input_.o("NAME","NEWPASSWORD2","TYPE","PASSWORD","SIZE",M_SIZE,0) << !td_ << !tr_ << "\n";
  CString RootEmail(Equivs.get_value(CGIStream, "ROOT-EMAIL", 0)); if (!RootEmail.StrLength()) RootEmail = "dblock@vestris.com";
  CGIStream << tr_ << td_.o("WIDTH",TD1WIDTH,"ALIGN","RIGHT", 0) << "new admin email: " << !td_ << td_ << input_.o("NAME","EMAIL","TYPE","TEXT","SIZE",M_SIZE,"VALUE",RootEmail.asString(),0) << !td_ << !tr_ << "\n";
  CString Blank(Equivs.get_value(CGIStream, "BLANK", 0)); if (!Blank.StrLength()) Blank = "/b.gif";
  CGIStream << tr_ << td_.o("WIDTH",TD1WIDTH,"ALIGN","RIGHT", 0) << "blank.gif (b.gif) URL: " << !td_ << td_ << input_.o("NAME","BLANK","TYPE","TEXT","SIZE",M_SIZE,"VALUE", Blank.asString(), 0) << !td_ << !tr_ << "\n";  	
  CString MailServer(Equivs.get_value(CGIStream, "MailServer", 0)); if (!MailServer.StrLength()) MailServer = "mail.vestris.com";
  CGIStream << tr_ << td_.o("WIDTH",TD1WIDTH,"ALIGN","RIGHT", 0) << "mail exchanger: " << !td_ << td_ << input_.o("NAME","MAILSERVER","TYPE","TEXT","SIZE",M_SIZE,"VALUE", MailServer.asString(), 0) << !td_ << !tr_ << "\n";  	
  CString ManageAdd(Equivs.get_value(CGIStream, "Manage-add", 0));
  CGIStream << tr_ << td_.o("WIDTH",TD1WIDTH,"ALIGN","RIGHT", 0) << "forum creation password: " << !td_ << td_ << input_.o("NAME","MANAGEADD","TYPE","TEXT","SIZE",M_SIZE,"VALUE", ManageAdd.asString(), 0) << !td_ << !tr_ << "\n";
  CGIStream << tr_ << td_.o("WIDTH",TD1WIDTH,"ALIGN","RIGHT", 0) << !td_ << td_ << input_.o("NAME","SUBMIT","TYPE","SUBMIT","VALUE","Generate Base", 0) << !td_ << !tr_ << "\n";
  CGIStream << !table_ << !form_ << "\n";
}
