#include "nextgen_v2.h"
#include "../inetsock/inetsock.h"
#include "../platform/platform.h"
#include "../platform/cgi_aid.h"
#include "../cstring/cstring.h"
#ifdef ANSI
#include <unistd.h>
#include <sys/time.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <fstream.h>
#endif

#include "../curl/curl.h"
#include "../cremotefile/cremotefile.h"

CString Proxy;

int nextgen_v2::line_count(void){
  int Result = 0;
  int iPos = contained.Pos('\n', displacement);
  while (iPos >= 0) {
    Result++;
    iPos = contained.Pos('\n', iPos+1);
  }
  return Result;
}

int nextgen_v2::Advance(const CString& item){
  int res = contained.isSubStr(item.asString(), displacement);
  if (res) {
    displacement = res;
    //contained.StrDelete(0, res);
    return 1;
  } else return 0;
}

CString nextgen_v2::check_dir(const CString& dir){
  #ifdef ANSI
  int head = dir.Pos('/');
  #else
  int head = dir.Pos('\\');
  #endif
  if (head > 0) {
    CString dir_short = dir.Copy(0, head);
    make_directory(dir_short);
    if (!chdir(dir_short.asString())) return(dir_short.append_backslash()+=check_dir(dir.Copy(head+1,dir.StrLength()))); 
    else return dir_short;
  } else return dir;
}

CString nextgen_v2::http_path(const CString& url){
  if (url.StartsWith("http://")) return(url.Copy(strlen("http://"), url.StrLength() - strlen("http://")));
  else return url;
}

void nextgen_v2::reopen(const CString& server, int port, const CString& document){
  displacement = 0;
  void_read = 0;
  fcontainer.Free();
  fserver = server;
  fport = port;
  fdocument = document;
  retreive_page(server, port, fdocument);
  if (!(contained.StrLength())) {void_read = 1; fs = 0;} else {fs = contained.StrLength();}
}

void nextgen_v2::reopen(const CString& container){
    void_read = 0;
    displacement = 0;
    fcontainer = container;
    buffer_read(container, fs);
    if (!(contained.StrLength())) void_read = 1;
}

void nextgen_v2::rewind(void){
  displacement = 0;
  //if (fcontainer.StrLength()) reopen(fcontainer);
  //else reopen(fserver, fport, fdocument);
}

long int nextgen_v2::nv2_fsize(void){
  return (fs);
}

nextgen_v2::nextgen_v2(const CString& container, const CString& Modified){
  DateModified = Modified;
  cache.Free();
  cached = 0;
  CString filename_stream;
#ifdef ANSI
  for (int i=0;i<container.StrLength();i++) if (container[i] != '\\') filename_stream+=container[i]; else if (container[i+1] == '\\') filename_stream+='\\';  
#else
  filename_stream = container;
#endif  
  common_nextgen_v2(filename_stream);
}

nextgen_v2::nextgen_v2(const CString& container){
  cache.Free();
  cached = 0;
  CString filename_stream;
#ifdef ANSI
  for (int i=0;i<container.StrLength();i++) if (container[i] != '\\') filename_stream+=container[i]; else if (container[i+1] == '\\') filename_stream+='\\';  
#else
  filename_stream = container;
#endif  
  common_nextgen_v2(filename_stream);
}

nextgen_v2::nextgen_v2(const CString& container, const CString& _cache, int _cached){
  cache = _cache;
  cached = _cached;  
  common_nextgen_v2(container);
}

void nextgen_v2::common_nextgen_v2(const CString& container){  
  Modified = 1;
  if (!container.StrLength()) return;
  if (container.StartsWith("http://")) {
    CUrl FUrl(container);    
    reopen(FUrl.GetHost(), FUrl.GetPortValue(), FUrl.GetUrlPath());
  } else reopen(container); // if doesn't start with http://
}

void nextgen_v2::common_nextgen_v2(const CString& server, int port, const CString& document){
  reopen(server, port, document);
}

nextgen_v2::nextgen_v2(const CString& server, int port, const CString& document){
  cache = 0;
  cached = 0;
  common_nextgen_v2(server, port, document);
}

nextgen_v2::nextgen_v2(const CString& server, int port, const CString& document, const CString& _cache, int _cached){
  cache = _cache;
  cached = _cached;
  common_nextgen_v2(server, port, document);
}

nextgen_v2::~nextgen_v2(void){
}

CString nextgen_v2::read_line(void) {
   CString Result;
   read_line(Result);
   return Result;
   //return contained.ExtractLine();
}

CString& nextgen_v2::read_line(CString& Result){
   //contained.ExtractLine(Target);
   int iPos = contained.Pos('\n', displacement);
   if (iPos >= 0) {
    contained.Copy(displacement, iPos-displacement, Result);
    displacement = ++iPos;
   } else {
    contained.Copy(displacement, contained.StrLength(), Result);    
    displacement = contained.StrLength();
   }
   return Result;
}

int nextgen_v2::feofc(void){
  if (!contained.StrLength()) return 1;
  else if (void_read) return 1;
  else if (displacement == contained.StrLength()) return 1;
  else return 0;
}
#define DEBUG

void nextgen_v2::buffer_read(const CString& filename, long int & fsize){
#define BUF_LEN 4096
  CString filename_stream;
  contained.Free();
  ifstream b_stream(filename.asString(), ios::in | ios::bin, filebuf::openprot);  
  if (!b_stream) { return; }
  char buf[BUF_LEN];
  int curPos = 0; char ch;  
  while(b_stream.get(ch)) {
    buf[curPos++] = ch;
    if (curPos == BUF_LEN-1) {
      buf[curPos] = 0;
      contained+=buf;
      curPos = 0;      
    }
  }
  if (curPos){
    buf[curPos] = 0;
    contained+=buf;
  }
  fsize = contained.StrLength();
  displacement = 0;
}

void nextgen_v2::retreive_page(const CString& _server, int port, const CString& _document){
  CString server(_server);
  CString document(_document);
  char current[MAXPATHLEN+1];
  getcwd(current, MAXPATHLEN);
  CString cached_complete;
  if (cache.StrLength() && (!chdir(cache.asString()))) {
    CString cached_document(_server);
    cached_document.append_backslash();
    cached_document+=document;
    cached_document =check_dir(cached_document);
    cached_complete = cache;
    cached_complete.append_backslash();
    cached_complete+=http_path(cached_document).remove_forward_slash();
    if (cached){
      reopen(cached_complete);
      if (!void_read){
	chdir(current);
	return;
      }	
    }
  }
 
  CString Full(_server);
  if (port) {
    Full+=":";
    Full+=port;
  }
  if (Full[Full.StrLength()-1]!='/') Full+='/';
  Full+=_document;  
  CRemoteFile FRemote(Full);
  FRemote.SetParameter("Pragma","no-cache");
  
  if (Proxy.StrLength()) FRemote.Get(Proxy); else FRemote.Get();
  contained = FRemote.GetData();
  if (FRemote.GetStatusValue() != 200) {
    CString Error("<h3>Error retrieving:</h3><b>");        
    Error+=Full;    
    Error+=" (";
    Error+=FRemote.GetStatusValue();
    Error+=")<br><h4>";
    Error+=wsLastError;    
    Error+="</h4>";
    contained = Error;
  } else {
    if (cached_complete.StrLength()) {
      ofstream cached_complete_ofstream(cached_complete.asString());
      cached_complete_ofstream << contained;
    }
  }
  chdir(current); 
}
